@@ -6,6 +6,7 @@ import glob
 import logging
 import os
 import sys
+from typing import Optional
 
 from galaxy.datatypes.metadata import MetadataElement
 from galaxy.datatypes.text import Html
@@ -27,7 +28,7 @@ class AnvioComposite(Html):
         cannot rename the datasets here - they come with the default unfortunately
         """
         defined_files = self.get_composite_files(dataset=dataset).items()
-        rval = ["<html><head><title>Files for Anvi'o Composite Dataset (%s)</title></head>" % (self.file_ext)]
+        rval = [f"<html><head><title>Files for Anvi'o Composite Dataset ({self.file_ext})</title></head>"]
         if defined_files:
             rval.append("<p/>This composite dataset is composed of the following defined files:<p/><ul>")
             for composite_name, composite_file in defined_files:
@@ -37,11 +38,11 @@ class AnvioComposite(Html):
                 missing_text = ''
                 if not os.path.exists(os.path.join(dataset.extra_files_path, composite_name)):
                     missing_text = ' (missing)'
-                rval.append('<li><a href="{}">{}</a>{}{}</li>'.format(composite_name, composite_name, opt_text, missing_text))
+                rval.append(f'<li><a href="{composite_name}">{composite_name}</a>{opt_text}{missing_text}</li>')
             rval.append("</ul>")
         defined_files = map(lambda x: x[0], defined_files)
         extra_files = []
-        for (dirpath, dirnames, filenames) in os.walk(dataset.extra_files_path, followlinks=True):
+        for dirpath, _dirnames, filenames in os.walk(dataset.extra_files_path, followlinks=True):
             for filename in filenames:
                 rel_path = os.path.relpath(os.path.join(dirpath, filename), dataset.extra_files_path)
                 if rel_path not in defined_files:
@@ -49,7 +50,7 @@ class AnvioComposite(Html):
         if extra_files:
             rval.append("<p/>This composite dataset contains these undefined files:<p/><ul>")
             for rel_path in extra_files:
-                rval.append('<li><a href="{}">{}</a></li>'.format(rel_path, rel_path))
+                rval.append(f'<li><a href="{rel_path}">{rel_path}</a></li>')
             rval.append('</ul>')
         if not (defined_files or extra_files):
             rval.append("<p/>This composite dataset does not contain any files!<p/><ul>")
@@ -79,11 +80,9 @@ class AnvioComposite(Html):
 
 class AnvioDB(AnvioComposite):
     """Class for AnvioDB database files."""
-    _anvio_basename = None
+    _anvio_basename: Optional[str] = None
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
     def __init__(self, *args, **kwd):
         super().__init__(*args, **kwd)
@@ -102,7 +101,7 @@ class AnvioDB(AnvioComposite):
             if found:
                 break
             if basename is not None and not os.path.exists(os.path.join(dataset.extra_files_path, basename)):
-                for name in glob.glob(os.path.join(dataset.extra_files_path, "*%s" % (basename))):
+                for name in glob.glob(os.path.join(dataset.extra_files_path, f"*{basename}")):
                     dataset.metadata.anvio_basename = os.path.basename(name)
                     found = True
                     break
@@ -113,8 +112,6 @@ class AnvioStructureDB(AnvioDB):
     _anvio_basename = 'STRUCTURE.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_structure_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
 
 class AnvioGenomesDB(AnvioDB):
@@ -122,8 +119,6 @@ class AnvioGenomesDB(AnvioDB):
     _anvio_basename = '-GENOMES.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_genomes_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
 
 class AnvioContigsDB(AnvioDB):
@@ -131,8 +126,6 @@ class AnvioContigsDB(AnvioDB):
     _anvio_basename = 'CONTIGS.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_contigs_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
     def __init__(self, *args, **kwd):
         super().__init__(*args, **kwd)
@@ -144,8 +137,6 @@ class AnvioProfileDB(AnvioDB):
     _anvio_basename = 'PROFILE.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_profile_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
     def __init__(self, *args, **kwd):
         super().__init__(*args, **kwd)
@@ -160,8 +151,6 @@ class AnvioPanDB(AnvioDB):
     _anvio_basename = 'PAN.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_pan_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
 
 class AnvioSamplesDB(AnvioDB):
@@ -169,8 +158,6 @@ class AnvioSamplesDB(AnvioDB):
     _anvio_basename = 'SAMPLES.db'
     MetadataElement(name="anvio_basename", default=_anvio_basename, desc="Basename", readonly=True)
     file_ext = 'anvio_samples_db'
-    composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
 
 if __name__ == '__main__':
