@@ -40,15 +40,15 @@ def __main__():
     try:
         chromCol = int(sys.argv[3]) - 1
     except Exception:
-        stop_err("'%s' is an invalid chrom column, correct the column settings before attempting to convert the data format." % str(sys.argv[3]))
+        stop_err(f"'{str(sys.argv[3])}' is an invalid chrom column, correct the column settings before attempting to convert the data format.")
     try:
         startCol = int(sys.argv[4]) - 1
     except Exception:
-        stop_err("'%s' is an invalid start column, correct the column settings before attempting to convert the data format." % str(sys.argv[4]))
+        stop_err(f"'{str(sys.argv[4])}' is an invalid start column, correct the column settings before attempting to convert the data format.")
     try:
         endCol = int(sys.argv[5]) - 1
     except Exception:
-        stop_err("'%s' is an invalid end column, correct the column settings before attempting to convert the data format." % str(sys.argv[5]))
+        stop_err(f"'{str(sys.argv[5])}' is an invalid end column, correct the column settings before attempting to convert the data format.")
     try:
         strandCol = int(sys.argv[6]) - 1
     except Exception:
@@ -119,7 +119,7 @@ def __main__():
                     break
                 if force_num_columns is not None and len(fields) != force_num_columns:
                     line = '\t'.join(force_bed_field_count(fields, count, force_num_columns))
-                out.write("%s\n" % line)
+                out.write(f"{line}\n")
     else:
         strict_bed = False
 
