@@ -23,8 +23,8 @@ def __main__():
                     spec_counts[spec] = 0
                 else:
                     spec_counts[spec] += 1
-                out.write("%s\n" % maf_utilities.get_fasta_header(c, {'block_index' : count, 'species' : spec, 'sequence_index' : spec_counts[spec]}, suffix="%s_%i_%i" % (spec, count, spec_counts[spec])))
-                out.write("%s\n" % c.text)
+                out.write("%s\n" % maf_utilities.get_fasta_header(c, {'block_index': count, 'species': spec, 'sequence_index': spec_counts[spec]}, suffix="%s_%i_%i" % (spec, count, spec_counts[spec])))
+                out.write(f"{c.text}\n")
             out.write("\n")
     print("%i MAF blocks converted to FASTA." % (count))
 
