@@ -26,7 +26,7 @@ def __main__():
                             out.write("%s\t%i\t%i\t%s\n" % (maf_utilities.src_split(c.src)[-1], c.get_forward_strand_start(), c.get_forward_strand_end(), c.strand))
                             count += 1
         except Exception as e:
-            print("There was a problem processing your input: %s" % e, file=sys.stderr)
+            print(f"There was a problem processing your input: {e}", file=sys.stderr)
     print("%i MAF blocks converted to Genomic Intervals for species %s." % (count, species))
 
 
