@@ -5,9 +5,8 @@ import os
 import shutil
 import string
 import tempfile
-import zipfile
-from collections import OrderedDict
 from inspect import isclass
+from typing import Any, Dict, Optional
 
 import webob.exc
 from markupsafe import escape
@@ -24,6 +23,7 @@ from galaxy.util import (
 )
 from galaxy.util.bunch import Bunch
 from galaxy.util.sanitize_html import sanitize_html
+from galaxy.util.zipstream import ZipstreamWrapper
 from . import (
     dataproviders,
     metadata
@@ -46,6 +46,10 @@ DOWNLOAD_FILENAME_PATTERN_COLLECTION_ELEMENT = "Galaxy${hdca_hid}-[${hdca_name}_
 DEFAULT_MAX_PEEK_SIZE = 1000000  # 1 MB
 
 
+class DatatypeConverterNotFoundException(Exception):
+    pass
+
+
 class DatatypeValidation:
 
     def __init__(self, state, message):
@@ -65,17 +69,36 @@ class DatatypeValidation:
         return DatatypeValidation("unknown", "Dataset validation unimplemented for this datatype.")
 
     def __repr__(self):
-        return "DatatypeValidation[state={},message={}]".format(self.state, self.message)
+        return f"DatatypeValidation[state={self.state},message={self.message}]"
 
 
 def validate(dataset_instance):
     try:
         datatype_validation = dataset_instance.datatype.validate(dataset_instance)
     except Exception as e:
-        datatype_validation = DatatypeValidation.invalid("Problem running datatype validation method [%s]" % str(e))
+        datatype_validation = DatatypeValidation.invalid(f"Problem running datatype validation method [{str(e)}]")
     return datatype_validation
 
 
+def get_params_and_input_name(converter, deps, target_context=None):
+    # Generate parameter dictionary
+    params = {}
+    # determine input parameter name and add to params
+    input_name = 'input1'
+    for key, value in converter.inputs.items():
+        if deps and value.name in deps:
+            params[value.name] = deps[value.name]
+        elif value.type == 'data':
+            input_name = key
+
+    # add potentially required/common internal tool parameters e.g. '__job_resource'
+    if target_context:
+        for key, value in target_context.items():
+            if key.startswith('__'):
+                params[key] = value
+    return params, input_name
+
+
 class DataMeta(abc.ABCMeta):
     """
     Metaclass for Data class.  Sets up metadata spec.
@@ -116,34 +139,44 @@ class Data(metaclass=DataMeta):
     # Add metadata elements
     MetadataElement(name="dbkey", desc="Database/Build", default="?", param=metadata.DBKeyParameter, multiple=False, no_value="?")
     # Stores the set of display applications, and viewing methods, supported by this datatype
-    supported_display_apps = {}
+    supported_display_apps: Dict[str, Any] = {}
     # If False, the peek is regenerated whenever a dataset of this type is copied
     copy_safe_peek = True
     # The dataset contains binary data --> do not space_to_tab or convert newlines, etc.
     # Allow binary file uploads of this type when True.
     is_binary = True
-    # Allow user to change between this datatype and others. If False, this datatype
-    # cannot be changed from or into.
-    allow_datatype_change = True
     # Composite datatypes
-    composite_type = None
-    composite_files = OrderedDict()
+    composite_type: Optional[str] = None
+    composite_files: Dict[str, Any] = {}
     primary_file_name = 'index'
+    # Allow user to change between this datatype and others. If left to None,
+    # datatype change is allowed if the datatype is not composite.
+    allow_datatype_change: Optional[bool] = None
     # A per datatype setting (inherited): max file size (in bytes) for setting optional metadata
     _max_optional_metadata_filesize = None
 
     # Trackster track type.
-    track_type = None
+    track_type: Optional[str] = None
 
     # Data sources.
-    data_sources = {}
+    data_sources: Dict[str, str] = {}
 
     def __init__(self, **kwd):
         """Initialize the datatype"""
         object.__init__(self, **kwd)
         self.supported_display_apps = self.supported_display_apps.copy()
         self.composite_files = self.composite_files.copy()
-        self.display_applications = OrderedDict()
+        self.display_applications = {}
+
+    @classmethod
+    def is_datatype_change_allowed(cls):
+        """
+        Returns the value of the `allow_datatype_change` class attribute if set
+        in a subclass, or True iff the datatype is not composite.
+        """
+        if cls.allow_datatype_change is not None:
+            return cls.allow_datatype_change
+        return cls.composite_type is None
 
     def get_raw_data(self, dataset):
         """Returns the full data. To stream it open the file_name and read/write as needed"""
@@ -170,7 +203,7 @@ class Data(metaclass=DataMeta):
         if copy_from:
             dataset.metadata = copy_from.metadata
 
-    def set_meta(self, dataset, overwrite=True, **kwd):
+    def set_meta(self, dataset: Any, overwrite=True, **kwd):
         """Unimplemented method, allows guessing of metadata from contents of file"""
         return True
 
@@ -234,11 +267,11 @@ class Data(metaclass=DataMeta):
                 line = line.strip()
                 if not line:
                     continue
-                out.append('<tr><td>%s</td></tr>' % escape(unicodify(line, 'utf-8')))
+                out.append(f"<tr><td>{escape(unicodify(line, 'utf-8'))}</td></tr>")
             out.append('</table>')
             out = "".join(out)
         except Exception as exc:
-            out = "Can't create peek: %s" % unicodify(exc)
+            out = f"Can't create peek: {unicodify(exc)}"
         return out
 
     def _archive_main_file(self, archive, display_name, data_filename):
@@ -251,9 +284,9 @@ class Data(metaclass=DataMeta):
         Returns a tuple of boolean, string, string: (error, msg, messagetype)
         """
         error, msg, messagetype = False, "", ""
-        archname = '%s.html' % display_name  # fake the real nature of the html file
+        archname = f'{display_name}.html'  # fake the real nature of the html file
         try:
-            archive.add(data_filename, archname)
+            archive.write(data_filename, archname)
         except OSError:
             error = True
             log.exception("Unable to add composite parent %s to temporary library download archive", data_filename)
@@ -265,74 +298,36 @@ class Data(metaclass=DataMeta):
         # save a composite object into a compressed archive for downloading
         outfname = data.name[0:150]
         outfname = ''.join(c in FILENAME_VALID_CHARS and c or '_' for c in outfname)
+        archive = ZipstreamWrapper(
+            archive_name=outfname,
+            upstream_mod_zip=trans.app.config.upstream_mod_zip,
+            upstream_gzip=trans.app.config.upstream_gzip
+        )
         error = False
         msg = ''
-        try:
-            if do_action == 'zip':
-                # Can't use mkstemp - the file must not exist first
-                tmpd = tempfile.mkdtemp(dir=trans.app.config.new_file_path, prefix='gx_composite_archive_')
-                util.umask_fix_perms(tmpd, trans.app.config.umask, 0o777, trans.app.config.gid)
-                tmpf = os.path.join(tmpd, 'library_download.' + do_action)
-                archive = zipfile.ZipFile(tmpf, 'w', zipfile.ZIP_DEFLATED, True)
+        ext = data.extension
+        path = data.file_name
+        efp = data.extra_files_path
+        # Add any central file to the archive,
 
-                def zipfile_add(fpath, arcname):
-                    encoded_arcname = arcname.encode('CP437')
-                    try:
-                        archive.write(fpath, encoded_arcname)
-                    except TypeError:
-                        # Despite documenting the need for CP437 encoded arcname,
-                        # python 3 actually needs this to be a unicode string ...
-                        # https://bugs.python.org/issue24110
-                        archive.write(fpath, arcname)
+        display_name = os.path.splitext(outfname)[0]
+        if not display_name.endswith(ext):
+            display_name = f'{display_name}_{ext}'
 
-                archive.add = zipfile_add
-
-            elif do_action == 'tgz':
-                archive = util.streamball.StreamBall('w|gz')
-            elif do_action == 'tbz':
-                archive = util.streamball.StreamBall('w|bz2')
-        except (OSError, zipfile.BadZipFile):
-            error = True
-            log.exception("Unable to create archive for download")
-            msg = "Unable to create archive for %s for download, please report this error" % outfname
+        error, msg = self._archive_main_file(archive, display_name, path)[:2]
         if not error:
-            ext = data.extension
-            path = data.file_name
-            efp = data.extra_files_path
-            # Add any central file to the archive,
-
-            display_name = os.path.splitext(outfname)[0]
-            if not display_name.endswith(ext):
-                display_name = '{}_{}'.format(display_name, ext)
-
-            error, msg = self._archive_main_file(archive, display_name, path)[:2]
-            if not error:
-                # Add any child files to the archive,
-                for fpath, rpath in self.__archive_extra_files_path(extra_files_path=efp):
-                    try:
-                        archive.add(fpath, rpath)
-                    except OSError:
-                        error = True
-                        log.exception("Unable to add %s to temporary library download archive", rpath)
-                        msg = "Unable to create archive for download, please report this error"
-                        continue
-            if not error:
-                if do_action == 'zip':
-                    archive.close()
-                    tmpfh = open(tmpf, 'rb')
-                    # CANNOT clean up - unlink/rmdir was always failing because file handle retained to return - must rely on a cron job to clean up tmp
-                    trans.response.set_content_type("application/x-zip-compressed")
-                    trans.response.headers["Content-Disposition"] = 'attachment; filename="%s.zip"' % outfname
-                    return tmpfh
-                else:
-                    trans.response.set_content_type("application/x-tar")
-                    outext = 'tgz'
-                    if do_action == 'tbz':
-                        outext = 'tbz'
-                    trans.response.headers["Content-Disposition"] = 'attachment; filename="{}.{}"'.format(outfname, outext)
-                    archive.wsgi_status = trans.response.wsgi_status()
-                    archive.wsgi_headeritems = trans.response.wsgi_headeritems()
-                    return archive.stream
+            # Add any child files to the archive,
+            for fpath, rpath in self.__archive_extra_files_path(extra_files_path=efp):
+                try:
+                    archive.write(fpath, rpath)
+                except OSError:
+                    error = True
+                    log.exception("Unable to add %s to temporary library download archive", rpath)
+                    msg = "Unable to create archive for download, please report this error"
+                    continue
+        if not error:
+            trans.response.headers.update(archive.get_headers())
+            return archive.response()
         return trans.show_error_message(msg)
 
     def __archive_extra_files_path(self, extra_files_path):
@@ -344,13 +339,13 @@ class Data(metaclass=DataMeta):
                 yield fpath, rpath
 
     def _serve_raw(self, trans, dataset, to_ext, **kwd):
-        trans.response.headers['Content-Length'] = int(os.stat(dataset.file_name).st_size)
+        trans.response.headers['Content-Length'] = str(os.stat(dataset.file_name).st_size)
         trans.response.set_content_type("application/octet-stream")  # force octet-stream so Safari doesn't append mime extensions to filename
-        filename = self._download_filename(dataset, to_ext, hdca=kwd.get("hdca"), element_identifier=kwd.get("element_identifier"))
-        trans.response.headers["Content-Disposition"] = 'attachment; filename="%s"' % filename
+        filename = self._download_filename(dataset, to_ext, hdca=kwd.get("hdca"), element_identifier=kwd.get("element_identifier"), filename_pattern=kwd.get("filename_pattern"))
+        trans.response.headers["Content-Disposition"] = f'attachment; filename="{filename}"'
         return open(dataset.file_name, mode='rb')
 
-    def to_archive(self, trans, dataset, name=""):
+    def to_archive(self, dataset, name=""):
         """
         Collect archive paths and file handles that need to be exported when archiving `dataset`.
 
@@ -358,19 +353,17 @@ class Data(metaclass=DataMeta):
         :param name: archive name, in collection context corresponds to collection name(s) and element_identifier,
                      joined by '/', e.g 'fastq_collection/sample1/forward'
         """
-        composite_extensions = trans.app.datatypes_registry.get_composite_extensions()
-        composite_extensions.append('html')  # for archiving composite datatypes
         rel_paths = []
         file_paths = []
-        if dataset.extension in composite_extensions:
-            main_file = "{}.{}".format(name, 'html')
+        if dataset.datatype.composite_type or dataset.extension == 'html':
+            main_file = f"{name}.html"
             rel_paths.append(main_file)
             file_paths.append(dataset.file_name)
             for fpath, rpath in self.__archive_extra_files_path(dataset.extra_files_path):
                 rel_paths.append(os.path.join(name, rpath))
                 file_paths.append(fpath)
         else:
-            rel_paths.append("{}.{}".format(name or dataset.file_name, dataset.extension))
+            rel_paths.append(f"{name or dataset.file_name}.{dataset.extension}")
             file_paths.append(dataset.file_name)
         return zip(file_paths, rel_paths)
 
@@ -416,7 +409,7 @@ class Data(metaclass=DataMeta):
                             # href = url_for(controller='dataset', action='display',
                             # dataset_id=trans.security.encode_id(data.dataset.id),
                             # preview=preview, filename=fname, to_ext=to_ext)
-                            tmp_fh.write('<tr bgcolor="{}"><td>{}</td></tr>\n'.format(bgcolor, escape(fname)))
+                            tmp_fh.write(f'<tr bgcolor="{bgcolor}"><td>{escape(fname)}</td></tr>\n')
                         tmp_fh.write('</table></body></html>\n')
                     return self._yield_user_file_content(trans, data, tmp_file_name)
                 mime = mimetypes.guess_type(file_path)[0]
@@ -428,22 +421,22 @@ class Data(metaclass=DataMeta):
                 self._clean_and_set_mime_type(trans, mime)
                 return self._yield_user_file_content(trans, data, file_path)
             else:
-                return webob.exc.HTTPNotFound("Could not find '{}' on the extra files path {}.".format(filename, file_path))
+                return webob.exc.HTTPNotFound(f"Could not find '{filename}' on the extra files path {file_path}.")
         self._clean_and_set_mime_type(trans, data.get_mime())
 
-        trans.log_event("Display dataset id: %s" % str(data.id))
+        trans.log_event(f"Display dataset id: {str(data.id)}")
         from galaxy import datatypes  # DBTODO REMOVE THIS AT REFACTOR
         if to_ext or isinstance(data.datatype, datatypes.binary.Binary):  # Saving the file, or binary file
             if data.extension in composite_extensions:
                 return self._archive_composite_dataset(trans, data, do_action=kwd.get('do_action', 'zip'))
             else:
-                trans.response.headers['Content-Length'] = int(os.stat(data.file_name).st_size)
-                filename = self._download_filename(data, to_ext, hdca=kwd.get("hdca"), element_identifier=kwd.get("element_identifier"))
+                trans.response.headers['Content-Length'] = str(os.stat(data.file_name).st_size)
+                filename = self._download_filename(data, to_ext, hdca=kwd.get("hdca"), element_identifier=kwd.get("element_identifier"), filename_pattern=kwd.get("filename_pattern"))
                 trans.response.set_content_type("application/octet-stream")  # force octet-stream so Safari doesn't append mime extensions to filename
-                trans.response.headers["Content-Disposition"] = 'attachment; filename="%s"' % filename
+                trans.response.headers["Content-Disposition"] = f'attachment; filename="{filename}"'
                 return open(data.file_name, 'rb')
         if not os.path.exists(data.file_name):
-            raise webob.exc.HTTPNotFound("File Not Found (%s)." % data.file_name)
+            raise webob.exc.HTTPNotFound(f"File Not Found ({data.file_name}).")
         max_peek_size = DEFAULT_MAX_PEEK_SIZE  # 1 MB
         if isinstance(data.datatype, datatypes.text.Html):
             max_peek_size = 10000000  # 10 MB for html
@@ -490,7 +483,7 @@ class Data(metaclass=DataMeta):
             # Sanitize anytime we respond with plain text/html content.
             # Check to see if this dataset's parent job is allowlisted
             # We cannot currently trust imported datasets for rendering.
-            if not from_dataset.creating_job.imported and from_dataset.creating_job.tool_id in trans.app.config.sanitize_allowlist:
+            if not from_dataset.creating_job.imported and from_dataset.creating_job.tool_id.startswith(tuple(trans.app.config.sanitize_allowlist)):
                 return open(filename, mode='rb')
 
             # This is returning to the browser, it needs to be encoded.
@@ -501,7 +494,7 @@ class Data(metaclass=DataMeta):
 
         return open(filename, mode='rb')
 
-    def _download_filename(self, dataset, to_ext, hdca=None, element_identifier=None):
+    def _download_filename(self, dataset, to_ext, hdca=None, element_identifier=None, filename_pattern=None):
         def escape(raw_identifier):
             return ''.join(c in FILENAME_VALID_CHARS and c or '_' for c in raw_identifier)[0:150]
 
@@ -516,7 +509,11 @@ class Data(metaclass=DataMeta):
             "hid": dataset.hid,
         }
 
-        filename_pattern = DOWNLOAD_FILENAME_PATTERN_DATASET
+        if not filename_pattern:
+            if hdca is None:
+                filename_pattern = DOWNLOAD_FILENAME_PATTERN_DATASET
+            else:
+                filename_pattern = DOWNLOAD_FILENAME_PATTERN_COLLECTION_ELEMENT
 
         if hdca is not None:
             # Use collection context to build up filename.
@@ -524,8 +521,6 @@ class Data(metaclass=DataMeta):
             template_values["hdca_name"] = escape(hdca.name)
             template_values["hdca_hid"] = hdca.hid
 
-            filename_pattern = DOWNLOAD_FILENAME_PATTERN_COLLECTION_ELEMENT
-
         return string.Template(filename_pattern).substitute(**template_values)
 
     def display_name(self, dataset):
@@ -592,7 +587,7 @@ class Data(metaclass=DataMeta):
         return self.display_applications.get(key, default)
 
     def get_display_applications_by_dataset(self, dataset, trans):
-        rval = OrderedDict()
+        rval = {}
         for key, value in self.display_applications.items():
             value = value.filter_by_dataset(dataset, trans)
             if value.links:
@@ -617,7 +612,7 @@ class Data(metaclass=DataMeta):
                 return getattr(self, self.supported_display_apps[type]['file_function'])(dataset, **kwd)
         except Exception:
             log.exception('Function %s is referred to in datatype %s for displaying as type %s, but is not accessible', self.supported_display_apps[type]['file_function'], self.__class__.__name__, type)
-        return "This display type ({}) is not implemented for this datatype ({}).".format(type, dataset.ext)
+        return f"This display type ({type}) is not implemented for this datatype ({dataset.ext})."
 
     def get_display_links(self, dataset, type, app, base_url, target_frame='_blank', **kwd):
         """
@@ -647,33 +642,24 @@ class Data(metaclass=DataMeta):
         converter = trans.app.datatypes_registry.get_converter_by_target_type(original_dataset.ext, target_type)
 
         if converter is None:
-            raise Exception("A converter does not exist for {} to {}.".format(original_dataset.ext, target_type))
-        # Generate parameter dictionary
-        params = {}
-        # determine input parameter name and add to params
-        input_name = 'input1'
-        for key, value in converter.inputs.items():
-            if deps and value.name in deps:
-                params[value.name] = deps[value.name]
-            elif value.type == 'data':
-                input_name = key
-        # add potentially required/common internal tool parameters e.g. '__job_resource'
-        if target_context:
-            for key, value in target_context.items():
-                if key.startswith('__'):
-                    params[key] = value
-        params[input_name] = original_dataset
+            raise DatatypeConverterNotFoundException(f"A converter does not exist for {original_dataset.ext} to {target_type}.")
 
+        params, input_name = get_params_and_input_name(converter, deps, target_context)
+
+        params[input_name] = original_dataset
+        # Make the target datatype available to the converter
+        params['__target_datatype__'] = target_type
         # Run converter, job is dispatched through Queue
-        converted_dataset = converter.execute(trans, incoming=params, set_output_hid=visible, history=history)[1]
+        job, converted_datasets, *_ = converter.execute(trans, incoming=params, set_output_hid=visible, history=history)
+        trans.app.job_manager.enqueue(job, tool=converter)
         if len(params) > 0:
-            trans.log_event("Converter params: %s" % (str(params)), tool_id=converter.id)
+            trans.log_event(f"Converter params: {str(params)}", tool_id=converter.id)
         if not visible:
-            for value in converted_dataset.values():
+            for value in converted_datasets.values():
                 value.visible = False
         if return_output:
-            return converted_dataset
-        return "The file conversion of {} on data {} has been added to the Queue.".format(converter.name, original_dataset.hid)
+            return converted_datasets
+        return f"The file conversion of {converter.name} on data {original_dataset.hid} has been added to the Queue."
 
     # We need to clear associated files before we set metadata
     # so that as soon as metadata starts to be set, e.g. implicitly converted datasets are deleted and no longer available 'while' metadata is being set, not just after
@@ -711,11 +697,11 @@ class Data(metaclass=DataMeta):
         return key
 
     @property
-    def writable_files(self, dataset=None):
-        files = OrderedDict()
+    def writable_files(self):
+        files = {}
         if self.composite_type != 'auto_primary_file':
             files[self.primary_file_name] = self.__new_composite_file(self.primary_file_name)
-        for key, value in self.get_composite_files(dataset=dataset).items():
+        for key, value in self.get_composite_files().items():
             files[key] = value
         return files
 
@@ -728,7 +714,7 @@ class Data(metaclass=DataMeta):
                     meta_value = self.metadata_spec[composite_file.substitute_name_with_metadata].default
                 return key % meta_value
             return key
-        files = OrderedDict()
+        files = {}
         for key, value in self.composite_files.items():
             files[substitute_composite_key(key, value)] = value
         return files
@@ -748,13 +734,14 @@ class Data(metaclass=DataMeta):
         datatype_classes = tuple(datatype if isclass(datatype) else datatype.__class__ for datatype in target_datatypes)
         return isinstance(self, datatype_classes)
 
+    @staticmethod
     def merge(split_files, output_file):
         """
             Merge files with copy.copyfileobj() will not hit the
             max argument limitation of cat. gz and bz2 files are also working.
         """
         if not split_files:
-            raise ValueError('Asked to merge zero files as %s' % output_file)
+            raise ValueError(f'Asked to merge zero files as {output_file}')
         elif len(split_files) == 1:
             shutil.copyfileobj(open(split_files[0], 'rb'), open(output_file, 'wb'))
         else:
@@ -762,8 +749,6 @@ class Data(metaclass=DataMeta):
                 for fsrc in split_files:
                     shutil.copyfileobj(open(fsrc, 'rb'), fdst)
 
-    merge = staticmethod(merge)
-
     def get_visualizations(self, dataset):
         """
         Returns a list of visualizations for datatype.
@@ -846,7 +831,7 @@ class Text(Data):
             sample_lines = dataset_read.count('\n')
             est_lines = int(sample_lines * (float(dataset.get_size()) / float(sample_size)))
         except UnicodeDecodeError:
-            log.error('Unable to estimate lines in file {}'.format(dataset.file_name))
+            log.error(f'Unable to estimate lines in file {dataset.file_name}')
             est_lines = None
         return est_lines
 
@@ -870,11 +855,11 @@ class Text(Data):
                     if line and not line.startswith('#'):
                         data_lines += 1
             except UnicodeDecodeError:
-                log.error('Unable to count lines in file {}'.format(dataset.file_name))
+                log.error(f'Unable to count lines in file {dataset.file_name}')
                 data_lines = None
         return data_lines
 
-    def set_peek(self, dataset, line_count=None, is_multi_byte=False, WIDTH=256, skipchars=None, line_wrap=True):
+    def set_peek(self, dataset, line_count=None, is_multi_byte=False, WIDTH=256, skipchars=None, line_wrap=True, **kwd):
         """
         Set the peek.  This method is used by various subclasses of Text.
         """
@@ -884,7 +869,7 @@ class Text(Data):
             if line_count is None:
                 # See if line_count is stored in the metadata
                 if dataset.metadata.data_lines:
-                    dataset.blurb = "{} {}".format(util.commaify(str(dataset.metadata.data_lines)), inflector.cond_plural(dataset.metadata.data_lines, self.line_class))
+                    dataset.blurb = f"{util.commaify(str(dataset.metadata.data_lines))} {inflector.cond_plural(dataset.metadata.data_lines, self.line_class)}"
                 else:
                     # Number of lines is not known ( this should not happen ), and auto-detect is
                     # needed to set metadata
@@ -894,21 +879,22 @@ class Text(Data):
                         lc = self.count_data_lines(dataset)
                         if lc is not None:
                             dataset.metadata.data_lines = lc
-                            dataset.blurb = "{} {}".format(util.commaify(str(lc)), inflector.cond_plural(lc, self.line_class))
+                            dataset.blurb = f"{util.commaify(str(lc))} {inflector.cond_plural(lc, self.line_class)}"
                         else:
                             dataset.blurb = "Error: Cannot count lines in dataset"
                     else:
                         est_lines = self.estimate_file_lines(dataset)
                         if est_lines is not None:
-                            dataset.blurb = "~{} {}".format(util.commaify(util.roundify(str(est_lines))), inflector.cond_plural(est_lines, self.line_class))
+                            dataset.blurb = f"~{util.commaify(util.roundify(str(est_lines)))} {inflector.cond_plural(est_lines, self.line_class)}"
                         else:
                             dataset.blurb = "Error: Cannot estimate lines in dataset"
             else:
-                dataset.blurb = "{} {}".format(util.commaify(str(line_count)), inflector.cond_plural(line_count, self.line_class))
+                dataset.blurb = f"{util.commaify(str(line_count))} {inflector.cond_plural(line_count, self.line_class)}"
         else:
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disk'
 
+    @classmethod
     def split(cls, input_datasets, subdir_generator_function, split_params):
         """
         Split the input files by line.
@@ -927,11 +913,8 @@ class Text(Data):
 
             # Computing the length is expensive!
             def _file_len(fname):
-                i = 0
                 with open(fname) as f:
-                    for i, _ in enumerate(f):
-                        pass
-                return i + 1
+                    return sum(1 for _ in f)
             length = _file_len(input_files[0])
             parts = int(split_params['split_size'])
             if length < parts:
@@ -947,7 +930,7 @@ class Text(Data):
         elif split_params['split_mode'] == 'to_size':
             chunk_size = int(split_params['split_size'])
         else:
-            raise Exception('Unsupported split mode %s' % split_params['split_mode'])
+            raise Exception(f"Unsupported split mode {split_params['split_mode']}")
 
         f = open(input_files[0])
         try:
@@ -980,7 +963,6 @@ class Text(Data):
             f.close()
             if part_file:
                 part_file.close()
-    split = classmethod(split)
 
     # ------------- Dataproviders
     @dataproviders.decorators.dataprovider_factory('line', dataproviders.line.FilteredLineDataProvider.settings)
@@ -1026,13 +1008,6 @@ class Newick(Text):
     edam_format = "format_1910"
     file_ext = "newick"
 
-    def __init__(self, **kwd):
-        """Initialize foobar datatype"""
-        Text.__init__(self, **kwd)
-
-    def init_meta(self, dataset, copy_from=None):
-        Text.init_meta(self, dataset, copy_from=copy_from)
-
     def sniff(self, filename):
         """ Returning false as the newick format is too general and cannot be sniffed."""
         return False
@@ -1041,7 +1016,6 @@ class Newick(Text):
         """
         Returns a list of visualizations for datatype.
         """
-
         return ['phyloviz']
 
 
@@ -1052,13 +1026,6 @@ class Nexus(Text):
     edam_format = "format_1912"
     file_ext = "nex"
 
-    def __init__(self, **kwd):
-        """Initialize foobar datatype"""
-        Text.__init__(self, **kwd)
-
-    def init_meta(self, dataset, copy_from=None):
-        Text.init_meta(self, dataset, copy_from=copy_from)
-
     def sniff_prefix(self, file_prefix):
         """All Nexus Files Simply puts a '#NEXUS' in its first line"""
         return file_prefix.string_io().read(6).upper() == "#NEXUS"
@@ -1067,7 +1034,6 @@ class Nexus(Text):
         """
         Returns a list of visualizations for datatype.
         """
-
         return ['phyloviz']
 
 
