@@ -15,9 +15,9 @@ import logging
 import os
 import re
 import sys
+from urllib.parse import quote_plus
 
 from markupsafe import escape
-from six.moves.urllib.parse import quote_plus
 
 from galaxy.datatypes import metadata
 from galaxy.datatypes.data import (
@@ -56,11 +56,11 @@ class GenomeGraphs(Tabular):
         """
         Initialize gg datatype, by adding UCSC display apps
         """
-        Tabular.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_display_app('ucsc', 'Genome Graph', 'as_ucsc_display_file', 'ucsc_links')
 
     def set_meta(self, dataset, **kwd):
-        Tabular.set_meta(self, dataset, **kwd)
+        super().set_meta(dataset, **kwd)
         dataset.metadata.markerCol = 1
         header = open(dataset.file_name).readlines()[0].strip().split('\t')
         dataset.metadata.columns = len(header)
@@ -102,26 +102,26 @@ class GenomeGraphs(Tabular):
                     internal_url = "%s" % app.url_for(controller='dataset',
                                                       dataset_id=dataset.id,
                                                       action='display_at',
-                                                      filename='ucsc_' + site_name)
+                                                      filename=f"ucsc_{site_name}")
                     display_url = "%s%s/display_as?id=%i&display_app=%s&authz_method=display_at" % (base_url, app.url_for(controller='root'), dataset.id, type)
                     display_url = quote_plus(display_url)
                     # was display_url = quote_plus( "%s/display_as?id=%i&display_app=%s" % (base_url, dataset.id, type) )
                     # redirect_url = quote_plus( "%sdb=%s&position=%s:%s-%s&hgt.customText=%%s" % (site_url, dataset.dbkey, chrom, start, stop) )
-                    sl = ["{}db={}".format(site_url, dataset.dbkey), ]
+                    sl = [f"{site_url}db={dataset.dbkey}", ]
                     # sl.append("&hgt.customText=%s")
-                    sl.append("&hgGenome_dataSetName={}&hgGenome_dataSetDescription={}".format(dataset.name, 'GalaxyGG_data'))
+                    sl.append(f"&hgGenome_dataSetName={dataset.name}&hgGenome_dataSetDescription=GalaxyGG_data")
                     sl.append("&hgGenome_formatType=best guess&hgGenome_markerType=best guess")
                     sl.append("&hgGenome_columnLabels=first row&hgGenome_maxVal=&hgGenome_labelVals=")
                     sl.append("&hgGenome_doSubmitUpload=submit")
-                    sl.append("&hgGenome_maxGapToFill=25000000&hgGenome_uploadFile=%s" % display_url)
+                    sl.append(f"&hgGenome_maxGapToFill=25000000&hgGenome_uploadFile={display_url}")
                     s = ''.join(sl)
                     s = quote_plus(s)
                     redirect_url = s
-                    link = '{}?redirect_url={}&display_url={}'.format(internal_url, redirect_url, display_url)
+                    link = f'{internal_url}?redirect_url={redirect_url}&display_url={display_url}'
                     ret_val.append((site_name, link))
         return ret_val
 
-    def make_html_table(self, dataset, skipchars=[]):
+    def make_html_table(self, dataset):
         """
         Create HTML table, used for displaying peek
         """
@@ -130,28 +130,28 @@ class GenomeGraphs(Tabular):
             with open(dataset.file_name) as f:
                 d = f.readlines()[:5]
             if len(d) == 0:
-                out = "Cannot find anything to parse in %s" % dataset.name
+                out = f"Cannot find anything to parse in {dataset.name}"
                 return out
             hasheader = 0
             try:
-                ['%f' % x for x in d[0][1:]]  # first is name - see if starts all numerics
+                [f'{x:f}' for x in d[0][1:]]  # first is name - see if starts all numerics
             except Exception:
                 hasheader = 1
             # Generate column header
             out.append('<tr>')
             if hasheader:
                 for i, name in enumerate(d[0].split()):
-                    out.append('<th>{}.{}</th>'.format(i + 1, name))
+                    out.append(f'<th>{i + 1}.{name}</th>')
                 d.pop(0)
                 out.append('</tr>')
             for row in d:
                 out.append('<tr>')
-                out.append(''.join('<td>%s</td>' % x for x in row.split()))
+                out.append(''.join(f'<td>{x}</td>' for x in row.split()))
                 out.append('</tr>')
             out.append('</table>')
             out = "".join(out)
         except Exception as exc:
-            out = "Can't create peek %s" % exc
+            out = f"Can't create peek {exc}"
         return out
 
     def validate(self, dataset, **kwd):
@@ -160,9 +160,9 @@ class GenomeGraphs(Tabular):
         """
         with open(dataset.file_name) as infile:
             next(infile)  # header
-            for i, row in enumerate(infile):
+            for row in infile:
                 ll = row.strip().split('\t')[1:]  # first is alpha feature identifier
-                for j, x in enumerate(ll):
+                for x in ll:
                     x = float(x)
         return DatatypeValidation.validated()
 
@@ -179,7 +179,7 @@ class GenomeGraphs(Tabular):
         True
         """
         buf = file_prefix.contents_header
-        rows = [l.split() for l in buf.splitlines()[1:4]]  # break on lines and drop header, small sample
+        rows = [line.split() for line in buf.splitlines()[1:4]]  # break on lines and drop header, small sample
 
         if len(rows) < 1:
             return False
@@ -214,7 +214,7 @@ class rgTabList(Tabular):
         """
         Initialize featurelistt datatype
         """
-        Tabular.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.column_names = []
 
     def display_peek(self, dataset):
@@ -240,7 +240,7 @@ class rgSampleList(rgTabList):
         """
         Initialize samplelist datatype
         """
-        rgTabList.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.column_names[0] = 'FID'
         self.column_names[1] = 'IID'
         # this is what Plink wants as at 2009
@@ -257,7 +257,7 @@ class rgFeatureList(rgTabList):
 
     def __init__(self, **kwd):
         """Initialize featurelist datatype"""
-        rgTabList.__init__(self, **kwd)
+        super().__init__(**kwd)
         for i, s in enumerate(['#FeatureId', 'Chr', 'Genpos', 'Mappos']):
             self.column_names[i] = s
 
@@ -273,7 +273,6 @@ class Rgenetics(Html):
                     readonly=True, set_in_upload=True)
 
     composite_type = 'auto_primary_file'
-    allow_datatype_change = False
     file_ext = 'rgenetics'
 
     def generate_primary_file(self, dataset=None):
@@ -285,9 +284,9 @@ class Rgenetics(Html):
             if composite_file.optional:
                 opt_text = ' (optional)'
             if composite_file.get('description'):
-                rval.append('<li><a href="{}" type="application/binary">{} ({})</a>{}</li>'.format(fn, fn, composite_file.get('description'), opt_text))
+                rval.append(f"<li><a href=\"{fn}\" type=\"application/binary\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
             else:
-                rval.append('<li><a href="{}" type="application/binary">{}</a>{}</li>'.format(fn, fn, opt_text))
+                rval.append(f'<li><a href="{fn}" type="application/binary">{fn}</a>{opt_text}</li>')
         rval.append('</ul></div></html>')
         return "\n".join(rval)
 
@@ -297,11 +296,11 @@ class Rgenetics(Html):
         """
         efp = dataset.extra_files_path
         flist = os.listdir(efp)
-        rval = ['<html><head><title>Files for Composite Dataset {}</title></head><body><p/>Composite {} contains:<p/><ul>'.format(dataset.name, dataset.name)]
-        for i, fname in enumerate(flist):
+        rval = [f'<html><head><title>Files for Composite Dataset {dataset.name}</title></head><body><p/>Composite {dataset.name} contains:<p/><ul>']
+        for fname in flist:
             sfname = os.path.split(fname)[-1]
             f, e = os.path.splitext(fname)
-            rval.append('<li><a href="{}">{}</a></li>'.format(sfname, sfname))
+            rval.append(f'<li><a href="{sfname}">{sfname}</a></li>')
         rval.append('</ul></body></html>')
         with open(dataset.file_name, 'w') as f:
             f.write("\n".join(rval))
@@ -316,7 +315,7 @@ class Rgenetics(Html):
         for lped/pbed eg
 
         """
-        Html.set_meta(self, dataset, **kwd)
+        super().set_meta(dataset, **kwd)
         if not kwd.get('overwrite'):
             if verbose:
                 gal_Log.debug('@@@ rgenetics set_meta called with overwrite = False')
@@ -325,17 +324,17 @@ class Rgenetics(Html):
             efp = dataset.extra_files_path
         except Exception:
             if verbose:
-                gal_Log.debug('@@@rgenetics set_meta failed {} - dataset {} has no efp ?'.format(sys.exc_info()[0], dataset.name))
+                gal_Log.debug(f'@@@rgenetics set_meta failed {sys.exc_info()[0]} - dataset {dataset.name} has no efp ?')
             return False
         try:
             flist = os.listdir(efp)
         except Exception:
             if verbose:
-                gal_Log.debug('@@@rgenetics set_meta failed {} - dataset {} has no efp ?'.format(sys.exc_info()[0], dataset.name))
+                gal_Log.debug(f'@@@rgenetics set_meta failed {sys.exc_info()[0]} - dataset {dataset.name} has no efp ?')
             return False
         if len(flist) == 0:
             if verbose:
-                gal_Log.debug('@@@rgenetics set_meta failed - {} efp {} is empty?'.format(dataset.name, efp))
+                gal_Log.debug(f'@@@rgenetics set_meta failed - {dataset.name} efp {efp} is empty?')
             return False
         self.regenerate_primary_file(dataset)
         if not dataset.info:
@@ -362,8 +361,8 @@ class SNPMatrix(Rgenetics):
     def sniff(self, filename):
         """ need to check the file header hex code
         """
-        infile = open(filename, "b")
-        head = infile.read(16)
+        with open(filename, "b") as infile:
+            head = infile.read(16)
         head = [hex(x) for x in head]
         if head != '':
             return False
@@ -378,7 +377,7 @@ class Lped(Rgenetics):
     file_ext = "lped"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.ped',
                                 description='Pedigree File',
                                 substitute_name_with_metadata='base_name',
@@ -396,7 +395,7 @@ class Pphe(Rgenetics):
     file_ext = "pphe"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.pphe',
                                 description='Plink Phenotype File',
                                 substitute_name_with_metadata='base_name',
@@ -411,7 +410,7 @@ class Fphe(Rgenetics):
     file_ext = "fphe"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.fphe',
                                 description='FBAT Phenotype File',
                                 substitute_name_with_metadata='base_name')
@@ -424,7 +423,7 @@ class Phe(Rgenetics):
     file_ext = "phe"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.phe',
                                 description='Phenotype File',
                                 substitute_name_with_metadata='base_name',
@@ -439,7 +438,7 @@ class Fped(Rgenetics):
     file_ext = "fped"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.fped', description='FBAT format pedfile',
                                 substitute_name_with_metadata='base_name',
                                 is_binary=False)
@@ -452,7 +451,7 @@ class Pbed(Rgenetics):
     file_ext = "pbed"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.bim', substitute_name_with_metadata='base_name', is_binary=False)
         self.add_composite_file('%s.bed', substitute_name_with_metadata='base_name', is_binary=True)
         self.add_composite_file('%s.fam', substitute_name_with_metadata='base_name', is_binary=False)
@@ -467,7 +466,7 @@ class ldIndep(Rgenetics):
     file_ext = "ldreduced"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.bim', substitute_name_with_metadata='base_name', is_binary=False)
         self.add_composite_file('%s.bed', substitute_name_with_metadata='base_name', is_binary=True)
         self.add_composite_file('%s.fam', substitute_name_with_metadata='base_name', is_binary=False)
@@ -482,7 +481,7 @@ class Eigenstratgeno(Rgenetics):
     file_ext = "eigenstratgeno"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.eigenstratgeno', substitute_name_with_metadata='base_name', is_binary=False)
         self.add_composite_file('%s.ind', substitute_name_with_metadata='base_name', is_binary=False)
         self.add_composite_file('%s.map', substitute_name_with_metadata='base_name', is_binary=False)
@@ -496,7 +495,7 @@ class Eigenstratpca(Rgenetics):
     file_ext = "eigenstratpca"
 
     def __init__(self, **kwd):
-        Rgenetics.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.eigenstratpca',
                                 description='Eigenstrat PCA file', substitute_name_with_metadata='base_name')
 
@@ -528,18 +527,17 @@ class IdeasPre(Html):
     MetadataElement(name="tmp_archive", desc="Compressed archive of compressed bed files", default=None, readonly=True)
 
     composite_type = 'auto_primary_file'
-    allow_datatype_change = False
     file_ext = 'ideaspre'
 
     def __init__(self, **kwd):
-        Html.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('chromosome_windows.txt', description='Chromosome window positions', is_binary=False, optional=True)
         self.add_composite_file('chromosomes.bed', description='Bed file specifying window positions', is_binary=False, optional=True)
         self.add_composite_file('IDEAS_input_config.txt', description='IDEAS input config', is_binary=False)
         self.add_composite_file('tmp.tar.gz', description='Compressed archive of compressed bed files', is_binary=True)
 
     def set_meta(self, dataset, **kwd):
-        Html.set_meta(self, dataset, **kwd)
+        super().set_meta(dataset, **kwd)
         for fname in os.listdir(dataset.extra_files_path):
             if fname.startswith("chromosomes"):
                 dataset.metadata.chrom_bed = os.path.join(dataset.extra_files_path, fname)
@@ -555,9 +553,9 @@ class IdeasPre(Html):
         rval = ['<html><head></head><body>']
         rval.append('<h3>Files prepared for IDEAS</h3>')
         rval.append('<ul>')
-        for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
+        for composite_name in self.get_composite_files(dataset=dataset).keys():
             fn = composite_name
-            rval.append('<li><a href="{}>{}</a></li>'.format(fn, fn))
+            rval.append(f'<li><a href="{fn}>{fn}</a></li>')
         rval.append('</ul></body></html>\n')
         return "\n".join(rval)
 
@@ -568,7 +566,7 @@ class IdeasPre(Html):
         rval.append('<ul>')
         for fname in os.listdir(dataset.extra_files_path):
             fn = os.path.split(fname)[-1]
-            rval.append('<li><a href="{}">{}</a></li>'.format(fn, fn))
+            rval.append(f'<li><a href="{fn}">{fn}</a></li>')
         rval.append('</ul></body></html>')
         with open(dataset.file_name, 'w') as f:
             f.write("\n".join(rval))
@@ -597,10 +595,9 @@ class RexpBase(Html):
     file_ext = 'rexpbase'
     html_table = None
     composite_type = 'auto_primary_file'
-    allow_datatype_change = False
 
     def __init__(self, **kwd):
-        Html.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.pheno', description='Phenodata tab text file',
                                 substitute_name_with_metadata='base_name', is_binary=False)
 
@@ -615,7 +612,7 @@ class RexpBase(Html):
         """Returns the mime type of the datatype"""
         return 'text/html'
 
-    def get_phecols(self, phenolist=[], maxConc=20):
+    def get_phecols(self, phenolist, maxConc=20):
         """
         sept 2009: cannot use whitespace to split - make a more complex structure here
         and adjust the methods that rely on this structure
@@ -662,7 +659,7 @@ class RexpBase(Html):
             for j in range(i + 1, nuse):
                 kdict = {}
                 for row in phe:  # row is a list of lists
-                    k = '{}{}'.format(row[i], row[j])  # composite key
+                    k = f'{row[i]}{row[j]}'  # composite key
                     kdict[k] = k
                 if (len(kdict.keys()) == len(concordance[useCols[j]])):  # i and j are always matched
                     delme.append(j)
@@ -710,12 +707,12 @@ class RexpBase(Html):
         note that R is weird and does not include the row.name in
         the header. why?"""
         if not dataset.dataset.purged:
-            pp = os.path.join(dataset.extra_files_path, '%s.pheno' % dataset.metadata.base_name)
+            pp = os.path.join(dataset.extra_files_path, f'{dataset.metadata.base_name}.pheno')
             try:
                 with open(pp) as f:
                     p = f.readlines()
             except Exception:
-                p = ['##failed to find %s' % pp, ]
+                p = [f'##failed to find {pp}', ]
             dataset.peek = ''.join(p[:5])
             dataset.blurb = 'Galaxy Rexpression composite file'
         else:
@@ -726,12 +723,12 @@ class RexpBase(Html):
         """
         expects a .pheno file in the extra_files_dir - ugh
         """
-        pp = os.path.join(dataset.extra_files_path, '%s.pheno' % dataset.metadata.base_name)
+        pp = os.path.join(dataset.extra_files_path, f'{dataset.metadata.base_name}.pheno')
         try:
             with open(pp) as f:
                 p = f.readlines()
         except Exception:
-            p = ['##failed to find %s' % pp]
+            p = [f'##failed to find {pp}']
         return ''.join(p[:5])
 
     def get_file_peek(self, filename):
@@ -752,10 +749,10 @@ class RexpBase(Html):
         """
         bn = dataset.metadata.base_name
         flist = os.listdir(dataset.extra_files_path)
-        rval = ['<html><head><title>Files for Composite Dataset %s</title></head><p/>Comprises the following files:<p/><ul>' % (bn)]
-        for i, fname in enumerate(flist):
+        rval = [f'<html><head><title>Files for Composite Dataset {bn}</title></head><p/>Comprises the following files:<p/><ul>']
+        for fname in flist:
             sfname = os.path.split(fname)[-1]
-            rval.append('<li><a href="{}">{}</a>'.format(sfname, sfname))
+            rval.append(f'<li><a href="{sfname}">{sfname}</a>')
         rval.append('</ul></html>')
         with open(dataset.file_name, 'w') as f:
             f.write("\n".join(rval))
@@ -771,7 +768,7 @@ class RexpBase(Html):
         from a BioC eSet or affybatch.
 
         """
-        Html.set_meta(self, dataset, **kwd)
+        super().set_meta(dataset, **kwd)
         try:
             flist = os.listdir(dataset.extra_files_path)
         except Exception:
@@ -787,7 +784,7 @@ class RexpBase(Html):
         if not bn:
             bn = '?'
             dataset.metadata.base_name = bn
-        pn = '%s.pheno' % (bn)
+        pn = f'{bn}.pheno'
         pp = os.path.join(dataset.extra_files_path, pn)
         dataset.metadata.pheno_path = pp
         try:
@@ -827,18 +824,18 @@ class RexpBase(Html):
             for i, row in enumerate(p):
                 lrow = row.strip().split('\t')
                 if i == 0:
-                    orow = ['<th>%s</th>' % escape(x) for x in lrow]
+                    orow = [f'<th>{escape(x)}</th>' for x in lrow]
                     orow.insert(0, '<tr>')
                     orow.append('</tr>')
                 else:
-                    orow = ['<td>%s</td>' % escape(x) for x in lrow]
+                    orow = [f'<td>{escape(x)}</td>' for x in lrow]
                     orow.insert(0, '<tr>')
                     orow.append('</tr>')
                 out.append(''.join(orow))
             out.append('</table>')
             out = "\n".join(out)
         except Exception as exc:
-            out = "Can't create html table %s" % unicodify(exc)
+            out = f"Can't create html table {unicodify(exc)}"
         return out
 
     def display_peek(self, dataset):
@@ -857,7 +854,7 @@ class Affybatch(RexpBase):
     file_ext = "affybatch"
 
     def __init__(self, **kwd):
-        RexpBase.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.affybatch',
                                 description='AffyBatch R object saved to file',
                                 substitute_name_with_metadata='base_name', is_binary=True)
@@ -870,7 +867,7 @@ class Eset(RexpBase):
     file_ext = "eset"
 
     def __init__(self, **kwd):
-        RexpBase.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.eset',
                                 description='ESet R object saved to file',
                                 substitute_name_with_metadata='base_name', is_binary=True)
@@ -883,7 +880,7 @@ class MAlist(RexpBase):
     file_ext = "malist"
 
     def __init__(self, **kwd):
-        RexpBase.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('%s.malist',
                                 description='MAlist R object saved to file',
                                 substitute_name_with_metadata='base_name', is_binary=True)
@@ -899,7 +896,7 @@ class LinkageStudies(Text):
     ]
 
     def __init__(self, **kwd):
-        Text.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.max_lines = 10
 
 
