@@ -34,7 +34,7 @@ class InfernalCM(Text):
             if dataset.metadata.number_of_models == 1:
                 dataset.blurb = "1 model"
             else:
-                dataset.blurb = "%s models" % dataset.metadata.number_of_models
+                dataset.blurb = f"{dataset.metadata.number_of_models} models"
             dataset.peek = get_file_peek(dataset.file_name, is_multi_byte=is_multi_byte)
         else:
             dataset.peek = 'file does not exist'
@@ -80,7 +80,7 @@ class Hmmer(Text):
         try:
             return dataset.peek
         except Exception:
-            return "HMMER database (%s)" % (nice_size(dataset.get_size()))
+            return f"HMMER database ({nice_size(dataset.get_size())})"
 
     @abc.abstractmethod
     def sniff_prefix(self, filename):
@@ -110,7 +110,6 @@ class Hmmer3(Hmmer):
 class HmmerPress(Binary):
     """Class for hmmpress database files."""
     file_ext = 'hmmpress'
-    allow_datatype_change = False
     composite_type = 'basic'
 
     def set_peek(self, dataset, is_multi_byte=False):
@@ -130,7 +129,7 @@ class HmmerPress(Binary):
             return "HMMER3 database (multiple files)"
 
     def __init__(self, **kwd):
-        Binary.__init__(self, **kwd)
+        super().__init__(**kwd)
         # Binary model
         self.add_composite_file('model.hmm.h3m', is_binary=True)
         # SSI index for binary model
@@ -154,7 +153,7 @@ class Stockholm_1_0(Text):
             if (dataset.metadata.number_of_models == 1):
                 dataset.blurb = "1 alignment"
             else:
-                dataset.blurb = "%s alignments" % dataset.metadata.number_of_models
+                dataset.blurb = f"{dataset.metadata.number_of_models} alignments"
             dataset.peek = get_file_peek(dataset.file_name)
         else:
             dataset.peek = 'file does not exist'
@@ -170,6 +169,7 @@ class Stockholm_1_0(Text):
         """
         dataset.metadata.number_of_models = generic_util.count_special_lines('^#[[:space:]+]STOCKHOLM[[:space:]+]1.0', dataset.file_name)
 
+    @classmethod
     def split(cls, input_datasets, subdir_generator_function, split_params):
         """
 
@@ -184,11 +184,11 @@ class Stockholm_1_0(Text):
 
         chunk_size = None
         if split_params['split_mode'] == 'number_of_parts':
-            raise Exception('Split mode "%s" is currently not implemented for STOCKHOLM-files.' % split_params['split_mode'])
+            raise Exception(f"Split mode \"{split_params['split_mode']}\" is currently not implemented for STOCKHOLM-files.")
         elif split_params['split_mode'] == 'to_size':
             chunk_size = int(split_params['split_size'])
         else:
-            raise Exception('Unsupported split mode %s' % split_params['split_mode'])
+            raise Exception(f"Unsupported split mode {split_params['split_mode']}")
 
         def _read_stockholm_records(filename):
             lines = []
@@ -219,7 +219,6 @@ class Stockholm_1_0(Text):
         except Exception as e:
             log.error('Unable to split files: %s', unicodify(e))
             raise
-    split = classmethod(split)
 
 
 @build_sniff_from_prefix
@@ -233,7 +232,7 @@ class MauveXmfa(Text):
             if (dataset.metadata.number_of_models == 1):
                 dataset.blurb = "1 alignment"
             else:
-                dataset.blurb = "%s alignments" % dataset.metadata.number_of_models
+                dataset.blurb = f"{dataset.metadata.number_of_models} alignments"
             dataset.peek = get_file_peek(dataset.file_name)
         else:
             dataset.peek = 'file does not exist'
@@ -244,3 +243,13 @@ class MauveXmfa(Text):
 
     def set_meta(self, dataset, **kwd):
         dataset.metadata.number_of_models = generic_util.count_special_lines('^#Sequence([[:digit:]]+)Entry', dataset.file_name)
+
+
+class Msf(Text):
+    """
+    Multiple sequence alignment format produced by the Accelrys GCG suite and
+    other programs.
+    """
+    edam_data = "data_0863"
+    edam_format = "format_1947"
+    file_ext = 'msf'
