@@ -22,11 +22,10 @@ class Wiff(Binary):
     edam_data = "data_2536"
     edam_format = "format_3710"
     file_ext = 'wiff'
-    allow_datatype_change = False
     composite_type = 'auto_primary_file'
 
     def __init__(self, **kwd):
-        Binary.__init__(self, **kwd)
+        super().__init__(**kwd)
 
         self.add_composite_file(
             'wiff',
@@ -47,9 +46,9 @@ class Wiff(Binary):
             if composite_file.optional:
                 opt_text = ' (optional)'
             if composite_file.get('description'):
-                rval.append('<li><a href="{}" type="text/plain">{} ({})</a>{}</li>'.format(fn, fn, composite_file.get('description'), opt_text))
+                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
             else:
-                rval.append('<li><a href="{}" type="text/plain">{}</a>{}</li>'.format(fn, fn, opt_text))
+                rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
         rval.append('</ul></div></html>')
         return "\n".join(rval)
 
@@ -323,7 +322,7 @@ class Dta(TabularData):
         data_lines = 0
         if dataset.has_data():
             with open(dataset.file_name) as dtafile:
-                for line in dtafile:
+                for _ in dtafile:
                     data_lines += 1
 
         # Guess column types
@@ -527,6 +526,7 @@ class Edta(TabularData):
     def set_meta(self, dataset, **kwd):
         data_lines = 0
         delim = None
+        tpe = None
         if dataset.has_data():
             with open(dataset.file_name) as dtafile:
                 for idx, line in enumerate(dtafile):
@@ -551,7 +551,7 @@ class Edta(TabularData):
         dataset.metadata.data_lines = data_lines
         dataset.metadata.comment_lines = 0
         dataset.metadata.columns = len(dataset.metadata.column_names)
-        if tpe > 0:
+        if tpe is not None and tpe > 0:
             dataset.metadata.comment_lines += 1
             dataset.metadata.data_lines -= 1
 
@@ -836,7 +836,7 @@ class ThermoRAW(Binary):
         try:
             return dataset.peek
         except Exception:
-            return "Thermo Finnigan RAW file (%s)" % (nice_size(dataset.get_size()))
+            return f"Thermo Finnigan RAW file ({nice_size(dataset.get_size())})"
 
 
 @build_sniff_from_prefix
@@ -881,7 +881,7 @@ class SPLib(Msp):
     composite_type = 'auto_primary_file'
 
     def __init__(self, **kwd):
-        Msp.__init__(self, **kwd)
+        super().__init__(**kwd)
         self.add_composite_file('library.splib',
                                 description='Spectral Library. Contains actual library spectra',
                                 is_binary=False)
@@ -899,9 +899,9 @@ class SPLib(Msp):
             if composite_file.optional:
                 opt_text = ' (optional)'
             if composite_file.get('description'):
-                rval.append('<li><a href="{}" type="text/plain">{} ({})</a>{}</li>'.format(fn, fn, composite_file.get('description'), opt_text))
+                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
             else:
-                rval.append('<li><a href="{}" type="text/plain">{}</a>{}</li>'.format(fn, fn, opt_text))
+                rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
         rval.append('</ul></div></html>')
         return "\n".join(rval)
 
@@ -943,7 +943,7 @@ class Ms2(Text):
         for header_field in ['CreationDate', 'Extractor', 'ExtractorVersion', 'ExtractorOptions']:
             found_header = False
             for header_line in header_lines:
-                if header_line.startswith('H\t%s' % (header_field)):
+                if header_line.startswith(f'H\t{header_field}'):
                     found_header = True
                     break
             if not found_header:
@@ -970,11 +970,10 @@ class ImzML(Binary):
     """
     edam_format = "format_3682"
     file_ext = 'imzml'
-    allow_datatype_change = False
     composite_type = 'auto_primary_file'
 
     def __init__(self, **kwd):
-        Binary.__init__(self, **kwd)
+        super().__init__(**kwd)
 
         """The metadata"""
         self.add_composite_file(
@@ -995,55 +994,8 @@ class ImzML(Binary):
             fn = composite_name
             opt_text = ''
             if composite_file.get('description'):
-                rval.append('<li><a href="{}" type="text/plain">{} ({})</a>{}</li>'.format(fn, fn, composite_file.get('description'), opt_text))
+                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
             else:
-                rval.append('<li><a href="{}" type="text/plain">{}</a>{}</li>'.format(fn, fn, opt_text))
-        rval.append('</ul></div></html>')
-        return "\n".join(rval)
-
-
-class Analyze75(Binary):
-    """
-        Mayo Analyze 7.5 files
-        http://www.imzml.org
-    """
-    file_ext = 'analyze75'
-    allow_datatype_change = False
-    composite_type = 'auto_primary_file'
-
-    def __init__(self, **kwd):
-        Binary.__init__(self, **kwd)
-
-        """The header file. Provides information about dimensions, identification, and processing history."""
-        self.add_composite_file(
-            'hdr',
-            description='The Analyze75 header file.',
-            is_binary=True)
-
-        """The image file.  Image data, whose data type and ordering are described by the header file."""
-        self.add_composite_file(
-            'img',
-            description='The Analyze75 image file.',
-            is_binary=True)
-
-        """The optional t2m file."""
-        self.add_composite_file(
-            't2m',
-            description='The Analyze75 t2m file.',
-            optional=True,
-            is_binary=True)
-
-    def generate_primary_file(self, dataset=None):
-        rval = ['<html><head><title>Analyze75 Composite Dataset.</title></head><p/>']
-        rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
-        for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
-            fn = composite_name
-            opt_text = ''
-            if composite_file.optional:
-                opt_text = ' (optional)'
-            if composite_file.get('description'):
-                rval.append('<li><a href="{}" type="text/plain">{} ({})</a>{}</li>'.format(fn, fn, composite_file.get('description'), opt_text))
-            else:
-                rval.append('<li><a href="{}" type="text/plain">{}</a>{}</li>'.format(fn, fn, opt_text))
+                rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
         rval.append('</ul></div></html>')
         return "\n".join(rval)
