@@ -3,20 +3,20 @@ File format detector
 """
 
 import bz2
-import codecs
 import gzip
 import io
 import logging
 import os
 import re
 import shutil
+import struct
 import sys
 import tempfile
 import urllib.request
 import zipfile
 
 from galaxy import util
-from galaxy.util import compression_utils
+from galaxy.util import compression_utils, stream_to_open_named_file
 from galaxy.util.checkers import (
     check_binary,
     check_bz2,
@@ -50,48 +50,16 @@ def stream_url_to_file(path, file_sources=None):
     prefix = "url_paste"
     if file_sources and file_sources.looks_like_uri(path):
         file_source_path = file_sources.get_file_source_path(path)
-        _, temp_name = tempfile.mkstemp(prefix=prefix)
-        os.close(_)
+        with tempfile.NamedTemporaryFile(prefix=prefix, delete=False) as temp:
+            temp_name = temp.name
         file_source_path.file_source.realize_to(file_source_path.path, temp_name)
         return temp_name
     else:
-        page = urllib.request.urlopen(path)  # page will be .close()ed in stream_to_file
+        page = urllib.request.urlopen(path, timeout=util.DEFAULT_SOCKET_TIMEOUT)  # page will be .close()ed in stream_to_file
         temp_name = stream_to_file(page, prefix=prefix, source_encoding=util.get_charset_from_http_headers(page.headers))
         return temp_name
 
 
-def stream_to_open_named_file(stream, fd, filename, source_encoding=None, source_error='strict', target_encoding=None, target_error='strict'):
-    """Writes a stream to the provided file descriptor, returns the file name. Closes file descriptor"""
-    # signature and behavor is somewhat odd, due to backwards compatibility, but this can/should be done better
-    CHUNK_SIZE = 1048576
-    try:
-        codecs.lookup(target_encoding)
-    except Exception:
-        target_encoding = util.DEFAULT_ENCODING  # utf-8
-    use_source_encoding = source_encoding is not None
-    while True:
-        chunk = stream.read(CHUNK_SIZE)
-        if not chunk:
-            break
-        if use_source_encoding:
-            # If a source encoding is given we use it to convert to the target encoding
-            try:
-                if not isinstance(chunk, str):
-                    chunk = chunk.decode(source_encoding, source_error)
-                os.write(fd, chunk.encode(target_encoding, target_error))
-            except UnicodeDecodeError:
-                use_source_encoding = False
-                os.write(fd, chunk)
-        else:
-            # Compressed files must be encoded after they are uncompressed in the upload utility,
-            # while binary files should not be encoded at all.
-            if isinstance(chunk, str):
-                chunk = chunk.encode(target_encoding, target_error)
-            os.write(fd, chunk)
-    os.close(fd)
-    return filename
-
-
 def stream_to_file(stream, suffix='', prefix='', dir=None, text=False, **kwd):
     """Writes a stream to a temporary file, returns the temporary file's name"""
     fd, temp_name = tempfile.mkstemp(suffix=suffix, prefix=prefix, dir=dir, text=text)
@@ -118,11 +86,10 @@ def convert_newlines(fname, in_place=True, tmp_dir=None, tmp_prefix="gxupload",
     Converts in place a file from universal line endings
     to Posix line endings.
     """
-    fd, temp_name = tempfile.mkstemp(prefix=tmp_prefix, dir=tmp_dir)
     i = 0
     NEWLINE_BYTE = 10
     CR_BYTE = 13
-    with open(fd, mode="wb") as fp, open(fname, mode="rb") as fi:
+    with tempfile.NamedTemporaryFile(mode='wb', prefix=tmp_prefix, dir=tmp_dir, delete=False) as fp, open(fname, mode='rb') as fi:
         last_char = None
         block = fi.read(block_size)
         last_block = b""
@@ -144,11 +111,11 @@ def convert_newlines(fname, in_place=True, tmp_dir=None, tmp_prefix="gxupload",
             i += 1
             fp.write(b"\n")
     if in_place:
-        shutil.move(temp_name, fname)
+        shutil.move(fp.name, fname)
         # Return number of lines in file.
         return (i, None)
     else:
-        return (i, temp_name)
+        return (i, fp.name)
 
 
 def convert_newlines_sep2tabs(fname, in_place=True, patt=br"[^\S\n]+", tmp_dir=None, tmp_prefix="gxupload"):
@@ -303,7 +270,7 @@ def guess_ext(fname, sniff_order, is_binary=False):
     >>> fname = get_test_fname('2.txt')
     >>> guess_ext(fname, sniff_order)
     'txt'
-    >>> fname = get_test_fname('2.tabular')
+    >>> fname = get_test_fname('test_tab2.tabular')
     >>> guess_ext(fname, sniff_order)
     'tabular'
     >>> fname = get_test_fname('3.txt')
@@ -422,7 +389,10 @@ def guess_ext(fname, sniff_order, is_binary=False):
     >>> fname = get_test_fname('test.blib')
     >>> guess_ext(fname, sniff_order)
     'blib'
-    >>> fname = get_test_fname('test.phylip')
+    >>> fname = get_test_fname('test_strict_interleaved.phylip')
+    >>> guess_ext(fname, sniff_order)
+    'phylip'
+    >>> fname = get_test_fname('test_relaxed_interleaved.phylip')
     >>> guess_ext(fname, sniff_order)
     'phylip'
     >>> fname = get_test_fname('1.smat')
@@ -437,6 +407,9 @@ def guess_ext(fname, sniff_order, is_binary=False):
     >>> fname = get_test_fname('1.phyloxml')
     >>> guess_ext(fname, sniff_order)
     'phyloxml'
+    >>> fname = get_test_fname('1.dzi')
+    >>> guess_ext(fname, sniff_order)
+    'dzi'
     >>> fname = get_test_fname('1.tiff')
     >>> guess_ext(fname, sniff_order)
     'tiff'
@@ -446,6 +419,12 @@ def guess_ext(fname, sniff_order, is_binary=False):
     >>> fname = get_test_fname('1.mtx')
     >>> guess_ext(fname, sniff_order)
     'mtx'
+    >>> fname = get_test_fname('mc_preprocess_summ.metacyto_summary.txt')
+    >>> guess_ext(fname, sniff_order)
+    'metacyto_summary.txt'
+    >>> fname = get_test_fname('Accuri_C6_A01_H2O.fcs')
+    >>> guess_ext(fname, sniff_order)
+    'fcs'
     >>> fname = get_test_fname('1imzml')
     >>> guess_ext(fname, sniff_order)  # This test case is ensuring doesn't throw exception, actual value could change if non-utf encoding handling improves.
     'data'
@@ -596,6 +575,19 @@ class FilePrefix:
     def search_str(self, query_str):
         return query_str in self.contents_header
 
+    def magic_header(self, pattern):
+        """
+        Unpack header and get first element
+        """
+        size = struct.calcsize(pattern)
+        header_bytes = self.contents_header_bytes[:size]
+        if len(header_bytes) < size:
+            return None
+        return struct.unpack(pattern, header_bytes)[0]
+
+    def startswith_bytes(self, test_bytes):
+        return self.contents_header_bytes.startswith(test_bytes)
+
 
 def build_sniff_from_prefix(klass):
     # Build and attach a sniff function to this class (klass) from the sniff_prefix function
@@ -676,21 +668,20 @@ def handle_compressed_file(
             keep_compressed = getattr(datatype, 'compressed', False)
     # don't waste time decompressing if we sniff invalid contents
     if is_compressed and is_valid and auto_decompress and not keep_compressed:
-        fd, uncompressed = tempfile.mkstemp(prefix=tmp_prefix, dir=tmp_dir)
-        compressed_file = DECOMPRESSION_FUNCTIONS[compressed_type](filename)
-        # TODO: it'd be ideal to convert to posix newlines and space-to-tab here as well
-        while True:
-            try:
-                chunk = compressed_file.read(CHUNK_SIZE)
-            except OSError as e:
-                os.close(fd)
-                os.remove(uncompressed)
-                compressed_file.close()
-                raise OSError('Problem uncompressing {} data, please try retrieving the data uncompressed: {}'.format(compressed_type, util.unicodify(e)))
-            if not chunk:
-                break
-            os.write(fd, chunk)
-        os.close(fd)
+        with tempfile.NamedTemporaryFile(prefix=tmp_prefix, dir=tmp_dir, delete=False) as uncompressed:
+            compressed_file = DECOMPRESSION_FUNCTIONS[compressed_type](filename)
+            # TODO: it'd be ideal to convert to posix newlines and space-to-tab here as well
+            while True:
+                try:
+                    chunk = compressed_file.read(CHUNK_SIZE)
+                except OSError as e:
+                    os.remove(uncompressed.name)
+                    compressed_file.close()
+                    raise OSError(f'Problem uncompressing {compressed_type} data, please try retrieving the data uncompressed: {util.unicodify(e)}')
+                if not chunk:
+                    break
+                uncompressed.write(chunk)
+        uncompressed = uncompressed.name
         compressed_file.close()
         if in_place:
             # Replace the compressed file with the uncompressed file
