@@ -18,7 +18,11 @@ from markupsafe import escape
 
 from galaxy import util
 from galaxy.datatypes import binary, data, metadata
-from galaxy.datatypes.metadata import MetadataElement
+from galaxy.datatypes.binary import _BamOrSam
+from galaxy.datatypes.metadata import (
+    MetadataElement,
+    MetadataParameter,
+)
 from galaxy.datatypes.sniff import (
     build_sniff_from_prefix,
     get_headers,
@@ -38,6 +42,7 @@ class TabularData(data.Text):
     # All tabular data is chunkable.
     CHUNKABLE = True
     data_line_offset = 0
+    max_peek_columns = 50
 
     """Add metadata elements"""
     MetadataElement(name="comment_lines", default=0, desc="Number of comment lines", readonly=False, optional=True, no_value=0)
@@ -51,10 +56,10 @@ class TabularData(data.Text):
     def set_meta(self, dataset, **kwd):
         raise NotImplementedError
 
-    def set_peek(self, dataset, line_count=None, is_multi_byte=False, WIDTH=256, skipchars=None):
-        super().set_peek(dataset, line_count=line_count, WIDTH=WIDTH, skipchars=skipchars, line_wrap=False)
+    def set_peek(self, dataset, line_count=None, is_multi_byte=False, WIDTH=256, skipchars=None, line_wrap=False, **kwd):
+        super().set_peek(dataset, line_count=line_count, WIDTH=WIDTH, skipchars=skipchars, line_wrap=line_wrap)
         if dataset.metadata.comment_lines:
-            dataset.blurb = "{}, {} comments".format(dataset.blurb, util.commaify(str(dataset.metadata.comment_lines)))
+            dataset.blurb = f"{dataset.blurb}, {util.commaify(str(dataset.metadata.comment_lines))} comments"
 
     def displayable(self, dataset):
         try:
@@ -136,7 +141,7 @@ class TabularData(data.Text):
             out.append('</table>')
             out = "".join(out)
         except Exception as exc:
-            out = "Can't create peek: %s" % util.unicodify(exc)
+            out = f"Can't create peek: {util.unicodify(exc)}"
         return out
 
     def make_html_peek_header(self, dataset, skipchars=None, column_names=None, column_number_format='%s', column_parameter_alias=None, **kwargs):
@@ -154,6 +159,7 @@ class TabularData(data.Text):
             columns = dataset.metadata.columns
             if columns is None:
                 columns = dataset.metadata.spec.columns.no_value
+            columns = min(columns, self.max_peek_columns)
             column_headers = [None] * columns
 
             # fill in empty headers with data from column_names
@@ -177,12 +183,12 @@ class TabularData(data.Text):
                 if header is None:
                     out.append(column_number_format % str(i + 1))
                 else:
-                    out.append('{}.{}'.format(str(i + 1), escape(header)))
+                    out.append(f'{str(i + 1)}.{escape(header)}')
                 out.append('</th>')
             out.append('</tr>')
         except Exception as exc:
             log.exception('make_html_peek_header failed on HDA %s', dataset.id)
-            raise Exception("Can't create peek header: %s" % util.unicodify(exc))
+            raise Exception(f"Can't create peek header: {util.unicodify(exc)}")
         return "".join(out)
 
     def make_html_peek_rows(self, dataset, skipchars=None, **kwargs):
@@ -198,26 +204,28 @@ class TabularData(data.Text):
             columns = dataset.metadata.columns
             if columns is None:
                 columns = dataset.metadata.spec.columns.no_value
+            columns = min(columns, self.max_peek_columns)
             for i, line in enumerate(peek.splitlines()):
                 if i >= self.data_line_offset:
                     if line.startswith(tuple(skipchars)):
-                        out.append('<tr><td colspan="100%%">%s</td></tr>' % escape(line))
+                        out.append(f'<tr><td colspan="100%">{escape(line)}</td></tr>')
                     elif line:
                         elems = line.split(dataset.metadata.delimiter)
+                        elems = elems[:min(len(elems), self.max_peek_columns)]
                         # pad shortened elems, since lines could have been truncated by width
                         if len(elems) < columns:
                             elems.extend([''] * (columns - len(elems)))
                         # we may have an invalid comment line or invalid data
                         if len(elems) != columns:
-                            out.append('<tr><td colspan="100%%">%s</td></tr>' % escape(line))
+                            out.append(f'<tr><td colspan="100%">{escape(line)}</td></tr>')
                         else:
                             out.append('<tr>')
                             for elem in elems:
-                                out.append('<td>%s</td>' % escape(elem))
+                                out.append(f'<td>{escape(elem)}</td>')
                             out.append('</tr>')
         except Exception as exc:
             log.exception('make_html_peek_rows failed on HDA %s', dataset.id)
-            raise Exception("Can't create peek rows: %s" % util.unicodify(exc))
+            raise Exception(f"Can't create peek rows: {util.unicodify(exc)}")
         return "".join(out)
 
     def display_peek(self, dataset):
@@ -256,6 +264,7 @@ class TabularData(data.Text):
 @dataproviders.decorators.has_dataproviders
 class Tabular(TabularData):
     """Tab delimited data"""
+    file_ext = "tabular"
 
     def get_column_names(self, first_line=None):
         return None
@@ -305,7 +314,7 @@ class Tabular(TabularData):
                 if column_type2 == column_type:
                     return False
             # neither column type was found in our ordered list, this cannot happen
-            raise ValueError("Tried to compare unknown column types: {} and {}".format(column_type1, column_type2))
+            raise ValueError(f"Tried to compare unknown column types: {column_type1} and {column_type2}")
 
         def is_int(column_text):
             # Don't allow underscores in numeric literals (PEP 515)
@@ -337,15 +346,17 @@ class Tabular(TabularData):
             if column_text == "":
                 return False
             return True
+
         is_column_type = {}  # Dict to store column type string to checking function
         for column_type in column_type_set_order:
-            is_column_type[column_type] = locals()["is_%s" % (column_type)]
+            is_column_type[column_type] = locals()[f"is_{column_type}"]
 
         def guess_column_type(column_text):
             for column_type in column_type_set_order:
                 if is_column_type[column_type](column_text):
                     return column_type
             return None
+
         data_lines = 0
         comment_lines = 0
         column_names = None
@@ -428,7 +439,7 @@ class Tabular(TabularData):
 
 class SraManifest(Tabular):
     """A manifest received from the sra_source tool."""
-    ext = 'sra_manifest.tabular'
+    file_ext = 'sra_manifest.tabular'
     data_line_offset = 1
 
     def set_meta(self, dataset, **kwds):
@@ -440,6 +451,8 @@ class SraManifest(Tabular):
 
 
 class Taxonomy(Tabular):
+    file_ext = "taxonomy"
+
     def __init__(self, **kwd):
         """Initialize taxonomy datatype"""
         super().__init__(**kwd)
@@ -456,13 +469,20 @@ class Taxonomy(Tabular):
 
 @dataproviders.decorators.has_dataproviders
 @build_sniff_from_prefix
-class Sam(Tabular):
+class Sam(Tabular, _BamOrSam):
     edam_format = "format_2573"
     edam_data = "data_0863"
     file_ext = 'sam'
     track_type = "ReadTrack"
     data_sources = {"data": "bam", "index": "bigwig"}
 
+    MetadataElement(name="bam_version", default=None, desc="BAM Version", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=None)
+    MetadataElement(name="sort_order", default=None, desc="Sort Order", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=None)
+    MetadataElement(name="read_groups", default=[], desc="Read Groups", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
+    MetadataElement(name="reference_names", default=[], desc="Chromosome Names", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
+    MetadataElement(name="reference_lengths", default=[], desc="Chromosome Lengths", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
+    MetadataElement(name="bam_header", default={}, desc="Dictionary of BAM Headers", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value={})
+
     def __init__(self, **kwd):
         """Initialize sam datatype"""
         super().__init__(**kwd)
@@ -531,21 +551,41 @@ class Sam(Tabular):
         return False
 
     def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=5, **kwd):
+        """
+        >>> from galaxy.datatypes.sniff import get_test_fname
+        >>> from galaxy.datatypes.registry import example_datatype_registry_for_sample
+        >>> from galaxy.model import Dataset, set_datatypes_registry
+        >>> from galaxy.model import History, HistoryDatasetAssociation
+        >>> from galaxy.model.mapping import init
+        >>> sa_session = init("/tmp", "sqlite:///:memory:", create_tables=True).session
+        >>> hist = History()
+        >>> sa_session.add(hist)
+        >>> sa_session.flush()
+        >>> set_datatypes_registry(example_datatype_registry_for_sample())
+        >>> fname = get_test_fname( 'sam_with_header.sam' )
+        >>> samds = Dataset(external_filename=fname)
+        >>> hda = hist.add_dataset(HistoryDatasetAssociation(id=1, extension='sam', create_dataset=True, sa_session=sa_session, dataset=samds))
+        >>> Sam().set_meta(hda)
+        >>> hda.metadata.comment_lines
+        2
+        >>> hda.metadata.reference_names
+        ['ref', 'ref2']
+        """
         if dataset.has_data():
             with open(dataset.file_name) as dataset_fh:
                 comment_lines = 0
                 if self.max_optional_metadata_filesize >= 0 and dataset.get_size() > self.max_optional_metadata_filesize:
                     # If the dataset is larger than optional_metadata, just count comment lines.
-                    for i, l in enumerate(dataset_fh):
-                        if l.startswith('@'):
+                    for line in dataset_fh:
+                        if line.startswith('@'):
                             comment_lines += 1
                         else:
-                            # No more comments, and the file is too big to look at the whole thing.  Give up.
+                            # No more comments, and the file is too big to look at the whole thing. Give up.
                             dataset.metadata.data_lines = None
                             break
                 else:
                     # Otherwise, read the whole thing and set num data lines.
-                    for i, l in enumerate(dataset_fh):
+                    for i, l in enumerate(dataset_fh):  # noqa: B007
                         if l.startswith('@'):
                             comment_lines += 1
                     dataset.metadata.data_lines = i + 1 - comment_lines
@@ -553,6 +593,9 @@ class Sam(Tabular):
             dataset.metadata.columns = 12
             dataset.metadata.column_types = ['str', 'int', 'str', 'int', 'int', 'str', 'str', 'int', 'int', 'str', 'str', 'str']
 
+            _BamOrSam().set_meta(dataset)
+
+    @staticmethod
     def merge(split_files, output_file):
         """
         Multiple SAM files may each have headers. Since the headers should all be the same, remove
@@ -564,8 +607,6 @@ class Sam(Tabular):
             cmd = ['egrep', '-v', '-h', '^@'] + split_files[1:] + ['>>', output_file]
             subprocess.check_call(cmd, shell=True)
 
-    merge = staticmethod(merge)
-
     # Dataproviders
     # sam does not use '#' to indicate comments/headers - we need to strip out those headers from the std. providers
     # TODO:?? seems like there should be an easier way to do this - metadata.comment_char?
@@ -681,7 +722,7 @@ class Pileup(Tabular):
         >>> fname = get_test_fname( '2.txt' )
         >>> Pileup().sniff( fname )  # 2.txt
         False
-        >>> fname = get_test_fname( '2.tabular' )
+        >>> fname = get_test_fname( 'test_tab2.tabular' )
         >>> Pileup().sniff( fname )
         False
         """
@@ -759,13 +800,13 @@ class BaseVcf(Tabular):
         stderr_f = tempfile.NamedTemporaryFile(prefix="bam_merge_stderr")
         stderr_name = stderr_f.name
         command = ["bcftools", "concat"] + split_files + ["-o", output_file]
-        log.info("Merging vcf files with command [%s]" % " ".join(command))
+        log.info(f"Merging vcf files with command [{' '.join(command)}]")
         exit_code = subprocess.call(args=command, stderr=open(stderr_name, 'wb'))
         with open(stderr_name, "rb") as f:
             stderr = f.read().strip()
         # Did merge succeed?
         if exit_code != 0:
-            raise Exception("Error merging VCF files: %s" % stderr)
+            raise Exception(f"Error merging VCF files: {stderr}")
 
     def validate(self, dataset, **kwd):
         def validate_row(row):
@@ -797,6 +838,7 @@ class Vcf(BaseVcf):
 class VcfGz(BaseVcf, binary.Binary):
     # This class name is a misnomer, should be VcfBgzip
     file_ext = 'vcf_bgzip'
+    file_ext_export_alias = 'vcf.gz'
     compressed = True
     compressed_format = "gzip"
 
@@ -822,9 +864,9 @@ class VcfGz(BaseVcf, binary.Binary):
             index_file = dataset.metadata.spec['tabix_index'].param.new_file(dataset=dataset)
 
         try:
-            pysam.tabix_index(dataset.file_name, index=index_file.file_name, preset='vcf', force=True)
+            pysam.tabix_index(dataset.file_name, index=index_file.file_name, preset='vcf', keep_original=True, force=True)
         except Exception as e:
-            raise Exception('Error setting VCF.gz metadata: %s' % (util.unicodify(e)))
+            raise Exception(f'Error setting VCF.gz metadata: {util.unicodify(e)}')
         dataset.metadata.tabix_index = index_file
 
 
@@ -860,17 +902,17 @@ class Eland(Tabular):
             # Generate column header
             out.append('<tr>')
             for i, name in enumerate(self.column_names):
-                out.append('<th>{}.{}</th>'.format(str(i + 1), name))
+                out.append(f'<th>{str(i + 1)}.{name}</th>')
             # This data type requires at least 11 columns in the data
             if dataset.metadata.columns - len(self.column_names) > 0:
-                for i in range(len(self.column_names), dataset.metadata.columns):
-                    out.append('<th>%s</th>' % str(i + 1))
+                for i in range(len(self.column_names), max(dataset.metadata.columns, self.max_peek_columns)):
+                    out.append(f'<th>{str(i + 1)}</th>')
                 out.append('</tr>')
             out.append(self.make_html_peek_rows(dataset, skipchars=skipchars, peek=peek))
             out.append('</table>')
             out = "".join(out)
         except Exception as exc:
-            out = "Can't create peek %s" % exc
+            out = f"Can't create peek {exc}"
         return out
 
     def sniff_prefix(self, file_prefix):
@@ -1033,7 +1075,7 @@ class BaseCSV(TabularData):
                     # No columns so not separated by this dialect.
                     return False
                 # ignore the length in the rest
-                for data_row in reader:
+                for _ in reader:
                     pass
 
         # Optional: Check Python's csv comes up with a similar dialect
@@ -1070,7 +1112,7 @@ class BaseCSV(TabularData):
                 try:
                     header_row = next(reader)
                     data_row = next(reader)
-                    for row in reader:
+                    for _ in reader:
                         pass
                 except StopIteration:
                     pass
@@ -1127,8 +1169,8 @@ class ConnectivityTable(Tabular):
     edam_format = "format_3309"
     file_ext = "ct"
 
-    header_regexp = re.compile("^[0-9]+" + "(?:\t|[ ]+)" + ".*?" + "(?:ENERGY|energy|dG)" + "[ \t].*?=")
-    structure_regexp = re.compile("^[0-9]+" + "(?:\t|[ ]+)" + "[ACGTURYKMSWBDHVN]+" + "(?:\t|[ ]+)" + "[^\t]+" + "(?:\t|[ ]+)" + "[^\t]+" + "(?:\t|[ ]+)" + "[^\t]+" + "(?:\t|[ ]+)" + "[^\t]+")
+    header_regexp = re.compile("^[0-9]+(?:	|[ ]+).*?(?:ENERGY|energy|dG)[ 	].*?=")
+    structure_regexp = re.compile("^[0-9]+(?:	|[ ]+)[ACGTURYKMSWBDHVN]+(?:	|[ ]+)[^	]+(?:	|[ ]+)[^	]+(?:	|[ ]+)[^	]+(?:	|[ ]+)[^	]+")
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
@@ -1139,8 +1181,9 @@ class ConnectivityTable(Tabular):
     def set_meta(self, dataset, **kwd):
         data_lines = 0
 
-        for line in open(dataset.file_name):
-            data_lines += 1
+        with open(dataset.file_name) as fh:
+            for _ in fh:
+                data_lines += 1
 
         dataset.metadata.data_lines = data_lines
 
@@ -1195,7 +1238,7 @@ class ConnectivityTable(Tabular):
                     else:
                         if j != int(re.split(r'\W+', line, 1)[0]):
                             return False
-                        elif j == length:  # Last line of first sequence has been recheached
+                        elif j == length:  # Last line of first sequence has been reached
                             return True
                         else:
                             j += 1
@@ -1224,7 +1267,7 @@ class ConnectivityTable(Tabular):
         ck_data_body = re.sub('\n[ \t]+', '\n', ck_data_body)
         ck_data_body = re.sub('[ ]+', '\t', ck_data_body)
 
-        return dumps({'ck_data': util.unicodify(ck_data_header + "\n" + ck_data_body), 'ck_index': ck_index + 1})
+        return dumps({'ck_data': util.unicodify(f"{ck_data_header}\n{ck_data_body}"), 'ck_index': ck_index + 1})
 
 
 @build_sniff_from_prefix
@@ -1235,18 +1278,20 @@ class MatrixMarket(TabularData):
     suitable for representing sparse matrices. Only nonzero entries need
     be encoded, and the coordinates of each are given explicitly.
 
-    The tabular file format is defined as follows::
+    The tabular file format is defined as follows:
 
-    %%MatrixMarket matrix coordinate real general <--- header line
-    %                                             <--+
-    % comments                                       |-- 0 or more comment lines
-    %                                             <--+
-        M  N  L                                   <--- rows, columns, entries
-        I1  J1  A(I1, J1)                         <--+
-        I2  J2  A(I2, J2)                            |
-        I3  J3  A(I3, J3)                            |-- L lines
-            . . .                                    |
-        IL JL  A(IL, JL)                          <--+
+    .. code-block::
+
+        %%MatrixMarket matrix coordinate real general <--- header line
+        %                                             <--+
+        % comments                                       |-- 0 or more comment lines
+        %                                             <--+
+            M  N  L                                   <--- rows, columns, entries
+            I1  J1  A(I1, J1)                         <--+
+            I2  J2  A(I2, J2)                            |
+            I3  J3  A(I3, J3)                            |-- L lines
+                . . .                                    |
+            IL JL  A(IL, JL)                          <--+
 
     Indices are 1-based, i.e. A(1,1) is the first element.
 
@@ -1269,23 +1314,116 @@ class MatrixMarket(TabularData):
         return file_prefix.startswith('%%MatrixMarket matrix coordinate')
 
     def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=5, **kwd):
+        if dataset.has_data():
+            # If the dataset is larger than optional_metadata, just count comment lines.
+            with open(dataset.file_name) as dataset_fh:
+                line = ''
+                data_lines = 0
+                comment_lines = 0
+                # If the dataset is larger than optional_metadata, just count comment lines.
+                count_comments_only = self.max_optional_metadata_filesize >= 0 and dataset.get_size() > self.max_optional_metadata_filesize
+                for line in dataset_fh:
+                    if line.startswith('%'):
+                        comment_lines += 1
+                    elif count_comments_only:
+                        data_lines = None
+                        break
+                    else:
+                        data_lines += 1
+                if ' ' in line:
+                    dataset.metadata.delimiter = ' '
+                else:
+                    dataset.metadata.delimiter = '\t'
+            dataset.metadata.comment_lines = comment_lines
+            dataset.metadata.data_lines = data_lines
+            dataset.metadata.columns = 3
+            dataset.metadata.column_types = ['int', 'int', 'float']
+
+
+@build_sniff_from_prefix
+class CMAP(TabularData):
+    MetadataElement(name="cmap_version", default='0.2', desc="version of cmap", readonly=True, visible=True, optional=False, no_value='0.2')
+    MetadataElement(name="label_channels", default=1, desc="the number of label channels", readonly=True, visible=True, optional=False, no_value=1)
+    MetadataElement(name="nickase_recognition_site_1", default=[], desc="comma separated list of label motif recognition sequences for channel 1", readonly=True, visible=True, optional=False, no_value=[])
+    MetadataElement(name="number_of_consensus_nanomaps", default=0, desc="the total number of consensus genome maps in the CMAP file", readonly=True, visible=True, optional=False, no_value=0)
+    MetadataElement(name="nickase_recognition_site_2", default=[], desc="comma separated list of label motif recognition sequences for channel 2", readonly=True, visible=True, optional=True, no_value=[])
+    MetadataElement(name="channel_1_color", default=[], desc="channel 1 color", readonly=True, visible=True, optional=True, no_value=[])
+    MetadataElement(name="channel_2_color", default=[], desc="channel 2 color", readonly=True, visible=True, optional=True, no_value=[])
+    """
+    # CMAP File Version:    2.0
+    # Label Channels:   1
+    # Nickase Recognition Site 1:   cttaag;green_01
+    # Nickase Recognition Site 2:   cctcagc;red_01
+    # Number of Consensus Maps: 459
+    # Values corresponding to intervals (StdDev, HapDelta) refer to the interval between current site and next site
+    #h  CMapId  ContigLength    NumSites    SiteID  LabelChannel    Position    StdDev  Coverage    Occurrence  ChimQuality SegDupL SegDupR FragileL    FragileR    OutlierFrac ChimNorm    Mask
+    #f  int float   int int int float   float   float   float   float   float   float   float   float   float   float   Hex
+    182 58474736.7  10235   1   1   58820.9 35.4    13.5    13.5    -1.00   -1.00   -1.00   3.63    0.00    0.00    -1.00   0
+    182 58474736.7  10235   1   1   58820.9 35.4    13.5    13.5    -1.00   -1.00   -1.00   3.63    0.00    0.00    -1.00   0
+    182 58474736.7  10235   1   1   58820.9 35.4    13.5    13.5    -1.00   -1.00   -1.00   3.63    0.00    0.00    -1.00   0
+    """
+    file_ext = "cmap"
+
+    def sniff_prefix(self, file_prefix):
+        handle = file_prefix.string_io()
+        for line in handle:
+            if not line.startswith('#'):
+                return False
+            if line.startswith('# CMAP File Version:'):
+                return True
+        return False
+
+    def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=7, **kwd):
         if dataset.has_data():
             with open(dataset.file_name) as dataset_fh:
                 comment_lines = 0
-                for i, l in enumerate(dataset_fh):
-                    if l.startswith('%'):
+                column_headers = None
+                cleaned_column_types = None
+                number_of_columns = 0
+                for i, line in enumerate(dataset_fh):
+                    line = line.strip('\n')
+                    if line.startswith('#'):
+
+                        if line.startswith('#h'):
+
+                            column_headers = line.split("\t")[1:]
+                        elif line.startswith('#f'):
+                            cleaned_column_types = []
+                            for column_type in line.split('\t')[1:]:
+                                if column_type == 'Hex':
+                                    cleaned_column_types.append('str')
+                                else:
+                                    cleaned_column_types.append(column_type)
                         comment_lines += 1
+                        fields = line.split('\t')
+                        if len(fields) == 2:
+                            if fields[0] == '# CMAP File Version:':
+                                dataset.metadata.cmap_version = fields[1]
+                            elif fields[0] == '# Label Channels:':
+                                dataset.metadata.label_channels = int(fields[1])
+                            elif fields[0] == '# Nickase Recognition Site 1:':
+                                fields2 = fields[1].split(';')
+                                if len(fields2) == 2:
+                                    dataset.metadata.channel_1_color = fields2[1]
+                                dataset.metadata.nickase_recognition_site_1 = fields2[0].split(',')
+                            elif fields[0] == '# Number of Consensus Maps:':
+                                dataset.metadata.number_of_consensus_nanomaps = int(fields[1])
+                            elif fields[0] == '# Nickase Recognition Site 2:':
+                                fields2 = fields[1].split(';')
+                                if len(fields2) == 2:
+                                    dataset.metadata.channel_2_color = fields2[1]
+                                dataset.metadata.nickase_recognition_site_2 = fields2[0].split(',')
                     elif self.max_optional_metadata_filesize >= 0 and dataset.get_size() > self.max_optional_metadata_filesize:
                         # If the dataset is larger than optional_metadata, just count comment lines.
                         # No more comments, and the file is too big to look at the whole thing. Give up.
                         dataset.metadata.data_lines = None
                         break
-                if ' ' in l:
-                    dataset.metadata.delimiter = ' '
-                else:
-                    dataset.metadata.delimiter = '\t'
+                    elif i == comment_lines + 1:
+                        number_of_columns = len(line.split('\t'))
                 if not (self.max_optional_metadata_filesize >= 0 and dataset.get_size() > self.max_optional_metadata_filesize):
                     dataset.metadata.data_lines = i + 1 - comment_lines
             dataset.metadata.comment_lines = comment_lines
-            dataset.metadata.columns = 3
-            dataset.metadata.column_types = ['int', 'int', 'float']
+            dataset.metadata.column_names = column_headers
+            dataset.metadata.column_types = cleaned_column_types
+            dataset.metadata.columns = number_of_columns
+            dataset.metadata.delimiter = '\t'
