@@ -10,6 +10,7 @@ from sys import getsizeof
 import numpy
 import sqlalchemy
 from sqlalchemy.ext.mutable import Mutable
+from sqlalchemy.inspection import inspect
 from sqlalchemy.types import (
     CHAR,
     LargeBinary,
@@ -77,7 +78,7 @@ class GalaxyLargeBinary(LargeBinary):
         return process
 
 
-class JSONType(sqlalchemy.types.TypeDecorator):
+class JSONType(TypeDecorator):
     """
     Represents an immutable structure as a json-encoded string.
 
@@ -89,6 +90,7 @@ class JSONType(sqlalchemy.types.TypeDecorator):
     # something like sqlalchemy.String, or even better, when applicable, native
     # sqlalchemy.dialects.postgresql.JSON
     impl = GalaxyLargeBinary
+    cache_ok = True
 
     def process_bind_param(self, value, dialect):
         if value is not None:
@@ -113,6 +115,10 @@ class JSONType(sqlalchemy.types.TypeDecorator):
         return (x == y)
 
 
+class MutableJSONType(JSONType):
+    """Associated with MutationObj"""
+
+
 class MutationObj(Mutable):
     """
     Mutable JSONType for SQLAlchemy from original gist:
@@ -123,6 +129,12 @@ class MutationObj(Mutable):
 
     And other minor changes to make it work for us.
     """
+
+    def __new__(cls, *args, **kwds):
+        self = super().__new__(cls, *args, **kwds)
+        self._key = None
+        return self
+
     @classmethod
     def coerce(cls, key, value):
         if isinstance(value, dict) and not isinstance(value, MutationDict):
@@ -146,15 +158,15 @@ class MutationObj(Mutable):
                 val = cls.coerce(key, val)
                 state.dict[key] = val
             if isinstance(val, cls):
-                val._parents[state.obj()] = key
+                val._parents[state] = key
 
         def set(target, value, oldvalue, initiator):
             if not isinstance(value, cls):
                 value = cls.coerce(key, value)
             if isinstance(value, cls):
-                value._parents[target.obj()] = key
+                value._parents[target] = key
             if isinstance(oldvalue, cls):
-                oldvalue._parents.pop(target.obj(), None)
+                oldvalue._parents.pop(inspect(target), None)
             return value
 
         def pickle(state, state_dict):
@@ -167,7 +179,7 @@ class MutationObj(Mutable):
         def unpickle(state, state_dict):
             if 'ext.mutable.values' in state_dict:
                 for val in state_dict['ext.mutable.values']:
-                    val._parents[state.obj()] = key
+                    val._parents[state] = key
 
         sqlalchemy.event.listen(parent_cls, 'load', load, raw=True, propagate=True)
         sqlalchemy.event.listen(parent_cls, 'refresh', load, raw=True, propagate=True)
@@ -185,13 +197,12 @@ class MutationDict(MutationObj, dict):
         return self
 
     def __setitem__(self, key, value):
-        if hasattr(self, '_key'):
-            value = MutationObj.coerce(self._key, value)
-        dict.__setitem__(self, key, value)
+        value = MutationObj.coerce(self._key, value)
+        super().__setitem__(key, value)
         self.changed()
 
     def __delitem__(self, key):
-        dict.__delitem__(self, key)
+        super().__delitem__(key)
         self.changed()
 
     def __getstate__(self):
@@ -200,6 +211,16 @@ class MutationDict(MutationObj, dict):
     def __setstate__(self, state):
         self.update(state)
 
+    def pop(self, *args, **kw):
+        value = super().pop(*args, **kw)
+        self.changed()
+        return value
+
+    def update(self, *args, **kwargs):
+        value = super().update(*args, **kwargs)
+        self.changed()
+        return value
+
 
 class MutationList(MutationObj, list):
     @classmethod
@@ -210,19 +231,19 @@ class MutationList(MutationObj, list):
         return self
 
     def __setitem__(self, idx, value):
-        list.__setitem__(self, idx, MutationObj.coerce(self._key, value))
+        super().__setitem__(idx, MutationObj.coerce(self._key, value))
         self.changed()
 
     def __setslice__(self, start, stop, values):
-        list.__setslice__(self, start, stop, (MutationObj.coerce(self._key, v) for v in values))
+        super().__setslice__(start, stop, (MutationObj.coerce(self._key, v) for v in values))
         self.changed()
 
     def __delitem__(self, idx):
-        list.__delitem__(self, idx)
+        super().__delitem__(idx)
         self.changed()
 
     def __delslice__(self, start, stop):
-        list.__delslice__(self, start, stop)
+        super().__delslice__(start, stop)
         self.changed()
 
     def __copy__(self):
@@ -232,37 +253,37 @@ class MutationList(MutationObj, list):
         return MutationList(MutationObj.coerce(self._key, copy.deepcopy(self[:])))
 
     def append(self, value):
-        list.append(self, MutationObj.coerce(self._key, value))
+        super().append(MutationObj.coerce(self._key, value))
         self.changed()
 
     def insert(self, idx, value):
-        list.insert(self, idx, MutationObj.coerce(self._key, value))
+        super().insert(self, idx, MutationObj.coerce(self._key, value))
         self.changed()
 
     def extend(self, values):
-        if hasattr(self, '_key'):
-            values = (MutationObj.coerce(self._key, value) for value in values)
-        list.extend(self, values)
+        values = (MutationObj.coerce(self._key, value) for value in values)
+        super().extend(values)
         self.changed()
 
     def pop(self, *args, **kw):
-        value = list.pop(self, *args, **kw)
+        value = super().pop(*args, **kw)
         self.changed()
         return value
 
     def remove(self, value):
-        list.remove(self, value)
+        super().remove(value)
         self.changed()
 
 
-MutationObj.associate_with(JSONType)
+MutationObj.associate_with(MutableJSONType)
+
 
 metadata_pickler = AliasPickleModule({
     ("cookbook.patterns", "Bunch"): ("galaxy.util.bunch", "Bunch")
 })
 
 
-def total_size(o, handlers={}, verbose=False):
+def total_size(o, handlers=None, verbose=False):
     """ Returns the approximate memory footprint an object and all of its contents.
 
     Automatically finds the contents of the following builtin containers and
@@ -274,6 +295,8 @@ def total_size(o, handlers={}, verbose=False):
 
     Recipe from:  https://code.activestate.com/recipes/577504-compute-memory-footprint-of-an-object-and-its-cont/
     """
+    handlers = handlers or {}
+
     def dict_handler(d):
         return chain.from_iterable(d.items())
 
@@ -315,7 +338,7 @@ class MetadataType(JSONType):
                     sz = total_size(v)
                     if sz > MAX_METADATA_VALUE_SIZE:
                         del value[k]
-                        log.warning('Refusing to bind metadata key {} due to size ({})'.format(k, sz))
+                        log.warning(f'Refusing to bind metadata key {k} due to size ({sz})')
             value = json_encoder.encode(value).encode()
         return value
 
@@ -345,6 +368,7 @@ class UUIDType(TypeDecorator):
     CHAR(32), storing as stringified hex values.
     """
     impl = CHAR
+    cache_ok = True
 
     def load_dialect_impl(self, dialect):
         return dialect.type_descriptor(CHAR(32))
@@ -366,6 +390,7 @@ class UUIDType(TypeDecorator):
 
 class TrimmedString(TypeDecorator):
     impl = String
+    cache_ok = True
 
     def process_bind_param(self, value, dialect):
         """Automatically truncate string values"""
