@@ -18,7 +18,7 @@ class DatasetCollectionTypesRegistry:
     def prototype(self, plugin_type):
         plugin_type_object = self.get(plugin_type)
         if not hasattr(plugin_type_object, 'prototype_elements'):
-            raise Exception("Cannot pre-determine structure for collection of type %s" % plugin_type)
+            raise Exception(f"Cannot pre-determine structure for collection of type {plugin_type}")
 
         dataset_collection = model.DatasetCollection()
         for e in plugin_type_object.prototype_elements():
