@@ -11,11 +11,14 @@ import sys
 import tempfile
 import weakref
 from collections import OrderedDict
+from collections.abc import Mapping
 from os.path import abspath
 
 from sqlalchemy.orm import object_session
+from sqlalchemy.orm.attributes import flag_modified
 
 import galaxy.model
+from galaxy.security.object_wrapper import sanitize_lists_to_string
 from galaxy.util import (
     form_builder,
     listify,
@@ -24,7 +27,6 @@ from galaxy.util import (
     unicodify,
 )
 from galaxy.util.json import safe_dumps
-from galaxy.util.object_wrapper import sanitize_lists_to_string
 
 log = logging.getLogger(__name__)
 
@@ -56,7 +58,7 @@ class Statement:
             statement.target(element, *args, **kwargs)  # statement.target is MetadataElementSpec, element is a Datatype class
 
 
-class MetadataCollection:
+class MetadataCollection(Mapping):
     """
     MetadataCollection is not a collection at all, but rather a proxy
     to the real metadata which is stored as a Dictionary. This class
@@ -87,16 +89,25 @@ class MetadataCollection:
         return self.parent.datatype.metadata_spec
 
     def __iter__(self):
-        return self.parent._metadata.__iter__()
+        yield from self.spec.keys()
 
-    def get(self, key, default=None):
+    def __getitem__(self, key):
         try:
-            return self.__getattr__(key) or default
-        except Exception:
-            return default
+            self.__getattribute__(key)
+        except AttributeError:
+            try:
+                return self.__getattr__(key)
+            except Exception:
+                raise KeyError
+        # `key` is an attribute of this instance, not some metadata: raise
+        # KeyError to prevent e.g. `'items' in dataset.metadata` from returning
+        # True
+        # Not doing this would also break Cheetah's NameMapper._valueForName()
+        # since dataset.metadata['items'] would be None
+        raise KeyError
 
-    def items(self):
-        return iter([(k, self.get(k)) for k in self.spec.keys()])
+    def __len__(self):
+        return len(self.spec)
 
     def __str__(self):
         return dict(self.items()).__str__()
@@ -112,6 +123,8 @@ class MetadataCollection:
             return self.spec[name].wrap(self.spec[name].default, object_session(self.parent))
         if name in self.parent._metadata:
             return self.parent._metadata[name]
+        # Instead of raising an AttributeError for non-existing metadata, we return None
+        return None
 
     def __setattr__(self, name, value):
         if name == "parent":
@@ -121,27 +134,29 @@ class MetadataCollection:
                 self.parent._metadata[name] = self.spec[name].unwrap(value)
             else:
                 self.parent._metadata[name] = value
+            flag_modified(self.parent, '_metadata')
 
     def remove_key(self, name):
         if name in self.parent._metadata:
             del self.parent._metadata[name]
         else:
-            log.info("Attempted to delete invalid key '%s' from MetadataCollection" % name)
+            log.info(f"Attempted to delete invalid key '{name}' from MetadataCollection")
 
     def element_is_set(self, name):
         """
         check if the meta data with the given name is set, i.e.
+
         - if the such a metadata actually exists and
         - if its value differs from no_value
 
-        param name the name of the metadata element
-        return True if the value differes from the no_value
-            False if its equal of if no metadata with the name is specified
+        :param name: the name of the metadata element
+        :returns: True if the value differes from the no_value
+                  False if its equal of if no metadata with the name is specified
         """
         try:
             meta_val = self.parent._metadata[name]
         except KeyError:
-            log.debug("no metadata with name %s found" % (name))
+            log.debug(f"no metadata with name {name} found")
             return False
 
         meta_spec = self.parent.metadata.spec[name]
@@ -172,17 +187,17 @@ class MetadataCollection:
     def from_JSON_dict(self, filename=None, path_rewriter=None, json_dict=None):
         dataset = self.parent
         if filename is not None:
-            log.debug('loading metadata from file for: {} {}'.format(dataset.__class__.__name__, dataset.id))
+            log.debug(f'loading metadata from file for: {dataset.__class__.__name__} {dataset.id}')
             with open(filename) as fh:
                 JSONified_dict = json.load(fh)
         elif json_dict is not None:
-            log.debug('loading metadata from dict for: {} {}'.format(dataset.__class__.__name__, dataset.id))
+            log.debug(f'loading metadata from dict for: {dataset.__class__.__name__} {dataset.id}')
             if isinstance(json_dict, str):
                 JSONified_dict = json.loads(json_dict)
             elif isinstance(json_dict, dict):
                 JSONified_dict = json_dict
             else:
-                raise ValueError("json_dict must be either a dictionary or a string, got %s." % (type(json_dict)))
+                raise ValueError(f"json_dict must be either a dictionary or a string, got {type(json_dict)}.")
         else:
             raise ValueError("You must provide either a filename or a json_dict")
 
@@ -213,6 +228,7 @@ class MetadataCollection:
             dataset.validated_state = JSONified_dict['__validated_state__']
         if '__validated_state_message__' in JSONified_dict:
             dataset.validated_state_message = JSONified_dict['__validated_state_message__']
+        flag_modified(dataset, '_metadata')
 
     def to_JSON_dict(self, filename=None):
         meta_dict = {}
@@ -226,7 +242,10 @@ class MetadataCollection:
             meta_dict['__validated_state__'] = dataset_meta_dict['__validated_state__']
         if '__validated_state_message__' in dataset_meta_dict:
             meta_dict['__validated_state_message__'] = dataset_meta_dict['__validated_state_message__']
-        encoded_meta_dict = galaxy.model.custom_types.json_encoder.encode(meta_dict)
+        try:
+            encoded_meta_dict = galaxy.model.custom_types.json_encoder.encode(meta_dict)
+        except Exception as e:
+            raise Exception(f"Failed encoding metadata dictionary: {meta_dict}") from e
         if filename is None:
             return encoded_meta_dict
         with open(filename, 'wt+') as fh:
@@ -540,7 +559,10 @@ class FileParameter(MetadataParameter):
             return None
         if isinstance(value, galaxy.model.MetadataFile) or isinstance(value, MetadataTempFile):
             return value
-        return session.query(galaxy.model.MetadataFile).get(value)
+        if isinstance(value, int):
+            return session.query(galaxy.model.MetadataFile).get(value)
+        else:
+            return session.query(galaxy.model.MetadataFile).filter_by(uuid=value).one()
 
     def make_copy(self, value, target_context, source_context):
         value = self.wrap(value, object_session(target_context.parent))
@@ -551,8 +573,11 @@ class FileParameter(MetadataParameter):
             # so this would ultimately get overwritten anyway.
             new_value = galaxy.model.MetadataFile(dataset=target_context.parent, name=self.spec.name)
             object_session(target_context.parent).add(new_value)
-            object_session(target_context.parent).flush()
-            shutil.copy(value.file_name, new_value.file_name)
+            try:
+                shutil.copy(value.file_name, new_value.file_name)
+            except AssertionError:
+                object_session(target_context.parent).flush()
+                shutil.copy(value.file_name, new_value.file_name)
             return self.unwrap(new_value)
         return None
 
@@ -563,6 +588,8 @@ class FileParameter(MetadataParameter):
             # as in extended_metadata mode, so there we just accept MetadataFile.
             # We will only serialize MetadataFile in this mode and not push to the database, so this is OK.
             value = value.id or value
+            if not isinstance(value, int) and object_session(value):
+                value = str(value.uuid)
         return value
 
     def from_external_value(self, value, parent, path_rewriter=None):
