@@ -21,7 +21,7 @@ def upgrade(migrate_engine):
         t = Table(t_name, metadata, autoload=True)
         c = Column("object_store_id", TrimmedString(255), index=True)
         try:
-            c.create(t, index_name="ix_%s_object_store_id" % t_name)
+            c.create(t, index_name=f"ix_{t_name}_object_store_id")
             assert c is t.c.object_store_id
         except Exception:
             log.exception("Adding object_store_id column to %s table failed.", t_name)
