@@ -1,6 +1,8 @@
 import logging
 import re
+from typing import Dict
 
+from sqlalchemy.orm.scoping import scoped_session
 from sqlalchemy.sql import select
 from sqlalchemy.sql.expression import func
 
@@ -26,7 +28,7 @@ class TagHandler:
     Manages CRUD operations related to tagging objects.
     """
 
-    def __init__(self, sa_session):
+    def __init__(self, sa_session: scoped_session) -> None:
         self.sa_session = sa_session
         # Minimum tag length.
         self.min_tag_len = 1
@@ -39,7 +41,7 @@ class TagHandler:
         # Key-value separator.
         self.key_value_separators = "=:"
         # Initialize with known classes - add to this in subclasses.
-        self.item_tag_assoc_info = {}
+        self.item_tag_assoc_info: Dict[str, ItemTagAssocInfo] = {}
 
     def create_tag_handler_session(self):
         # Creates a transient tag handler that avoids repeated flushes
@@ -154,6 +156,8 @@ class TagHandler:
 
     def apply_item_tag(self, user, item, name, value=None, flush=True):
         # Use lowercase name for searching/creating tag.
+        if name is None:
+            return
         lc_name = name.lower()
         # Get or create item-tag association.
         item_tag_assoc = self._get_item_tag_assoc(user, item, lc_name)
@@ -164,7 +168,7 @@ class TagHandler:
             # Create tag; if None, skip the tag (and log error).
             tag = self._get_or_create_tag(lc_name)
             if not tag:
-                log.warning("Failed to create tag with name %s" % lc_name)
+                log.warning(f"Failed to create tag with name {lc_name}")
                 return
             # Create tag association based on item class.
             item_tag_assoc_class = self.get_tag_assoc_class(item.__class__)
@@ -202,7 +206,7 @@ class TagHandler:
         for tag in tags:
             tag_str = tag.user_tname
             if tag.value is not None:
-                tag_str += ":" + tag.user_value
+                tag_str += f":{tag.user_value}"
             tags_str_list.append(tag_str)
         return ", ".join(tags_str_list)
 
@@ -216,22 +220,25 @@ class TagHandler:
             return self.sa_session.query(galaxy.model.Tag).filter_by(name=tag_name.lower()).first()
         return None
 
-    def _create_tag(self, tag_str):
+    def _create_tag(self, tag_str: str):
         """Create a Tag object from a tag string."""
         tag_hierarchy = tag_str.split(self.hierarchy_separator)
         tag_prefix = ""
         parent_tag = None
+        tag = None
         for sub_tag in tag_hierarchy:
             # Get or create subtag.
-            tag_name = tag_prefix + self._scrub_tag_name(sub_tag)
-            tag = self._get_tag(tag_name)
-            if not tag:
-                tag = self._create_tag_instance(tag_name)
-            # Set tag parent.
-            tag.parent = parent_tag
-            # Update parent and tag prefix.
-            parent_tag = tag
-            tag_prefix = tag.name + self.hierarchy_separator
+            sub_tag_name = self._scrub_tag_name(sub_tag)
+            if sub_tag_name:
+                tag_name = tag_prefix + sub_tag_name
+                tag = self._get_tag(tag_name)
+                if not tag:
+                    tag = self._create_tag_instance(tag_name)
+                # Set tag parent.
+                tag.parent = parent_tag
+                # Update parent and tag prefix.
+                parent_tag = tag
+                tag_prefix = tag.name + self.hierarchy_separator
         return tag
 
     def _get_tag(self, tag_name):
@@ -279,7 +286,7 @@ class TagHandler:
         # Strip unicode control characters
         tag_str = strip_control_characters(tag_str)
         # Split tags based on separators.
-        reg_exp = re.compile('[' + self.tag_separators + ']')
+        reg_exp = re.compile(f"[{self.tag_separators}]")
         raw_tags = reg_exp.split(tag_str)
         return self.parse_tags_list(raw_tags)
 
@@ -338,8 +345,8 @@ class TagHandler:
         """Get name, value pair from a tag string."""
         # Use regular expression to parse name, value.
         if tag_str.startswith('#'):
-            tag_str = "name:%s" % tag_str[1:]
-        reg_exp = re.compile("[" + self.key_value_separators + "]")
+            tag_str = f"name:{tag_str[1:]}"
+        reg_exp = re.compile(f"[{self.key_value_separators}]")
         name_value_pair = reg_exp.split(tag_str, 1)
         # Add empty slot if tag does not have value.
         if len(name_value_pair) < 2:
@@ -348,37 +355,38 @@ class TagHandler:
 
 
 class GalaxyTagHandler(TagHandler):
-    def __init__(self, sa_session):
+    def __init__(self, sa_session: scoped_session):
         from galaxy import model
         TagHandler.__init__(self, sa_session)
         self.item_tag_assoc_info["History"] = ItemTagAssocInfo(model.History,
                                                                model.HistoryTagAssociation,
-                                                               model.HistoryTagAssociation.table.c.history_id)
+                                                               model.HistoryTagAssociation.history_id)
         self.item_tag_assoc_info["HistoryDatasetAssociation"] = \
             ItemTagAssocInfo(model.HistoryDatasetAssociation,
                              model.HistoryDatasetAssociationTagAssociation,
-                             model.HistoryDatasetAssociationTagAssociation.table.c.history_dataset_association_id)
+                             model.HistoryDatasetAssociationTagAssociation.history_dataset_association_id)
         self.item_tag_assoc_info["HistoryDatasetCollectionAssociation"] = \
             ItemTagAssocInfo(model.HistoryDatasetCollectionAssociation,
                              model.HistoryDatasetCollectionTagAssociation,
-                             model.HistoryDatasetCollectionTagAssociation.table.c.history_dataset_collection_id)
+                             model.HistoryDatasetCollectionTagAssociation.history_dataset_collection_id)
         self.item_tag_assoc_info["LibraryDatasetDatasetAssociation"] = \
             ItemTagAssocInfo(model.LibraryDatasetDatasetAssociation,
                              model.LibraryDatasetDatasetAssociationTagAssociation,
-                             model.LibraryDatasetDatasetAssociationTagAssociation.table.c.library_dataset_dataset_association_id)
+                             model.LibraryDatasetDatasetAssociationTagAssociation.library_dataset_dataset_association_id)
         self.item_tag_assoc_info["Page"] = ItemTagAssocInfo(model.Page,
                                                             model.PageTagAssociation,
-                                                            model.PageTagAssociation.table.c.page_id)
+                                                            model.PageTagAssociation.page_id)
         self.item_tag_assoc_info["StoredWorkflow"] = ItemTagAssocInfo(model.StoredWorkflow,
                                                                       model.StoredWorkflowTagAssociation,
-                                                                      model.StoredWorkflowTagAssociation.table.c.stored_workflow_id)
+                                                                      model.StoredWorkflowTagAssociation.stored_workflow_id)
         self.item_tag_assoc_info["Visualization"] = ItemTagAssocInfo(model.Visualization,
                                                                      model.VisualizationTagAssociation,
-                                                                     model.VisualizationTagAssociation.table.c.visualization_id)
+                                                                     model.VisualizationTagAssociation.visualization_id)
 
 
 class GalaxyTagHandlerSession(GalaxyTagHandler):
     """Like GalaxyTagHandler, but avoids one flush per created tag."""
+
     def __init__(self, sa_session):
         super().__init__(sa_session)
         self.created_tags = {}
@@ -395,6 +403,17 @@ class GalaxyTagHandlerSession(GalaxyTagHandler):
         return tag
 
 
+class GalaxySessionlessTagHandler(GalaxyTagHandlerSession):
+
+    def _get_tag(self, tag_name):
+        """Get tag from cache or database."""
+        # Short-circuit session access
+        return self.created_tags.get(tag_name)
+
+    def get_tag_by_name(self, tag_name):
+        self.created_tags.get(tag_name)
+
+
 class CommunityTagHandler(TagHandler):
     def __init__(self, sa_session):
         TagHandler.__init__(self, sa_session)
