@@ -12,11 +12,8 @@ from sqlalchemy import (
     Table
 )
 from sqlalchemy.exc import NoSuchTableError
-from sqlalchemy_utils import (
-    create_database,
-    database_exists,
-)
 
+from galaxy.model.database_utils import create_database, database_exists
 from galaxy.model.tool_shed_install import mapping
 
 
@@ -27,12 +24,13 @@ migrate_repository_directory = os.path.abspath(os.path.dirname(__file__)).replac
 migrate_repository = repository.Repository(migrate_repository_directory)
 
 
-def create_or_verify_database(url, engine_options={}, app=None):
+def create_or_verify_database(url, engine_options=None, app=None):
     """
     """
     # Create engine and metadata
+    engine_options = engine_options or {}
     if not database_exists(url):
-        message = "Creating database for URI [%s]" % url
+        message = f"Creating database for URI [{url}]"
         log.info(message)
         create_database(url)
 
@@ -96,7 +94,7 @@ def migrate_to_current_version(engine, schema):
     changeset = schema.changeset(None)
     for ver, change in changeset:
         nextver = ver + changeset.step
-        log.info('Migrating {} -> {}... '.format(ver, nextver))
+        log.info(f'Migrating {ver} -> {nextver}... ')
         old_stdout = sys.stdout
 
         class FakeStdout:
