@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-
-__version__ = '20.9.1'
+__version__ = "21.9.0"
 
 PROJECT_NAME = "galaxy-data"
 PROJECT_OWNER = PROJECT_USERAME = "galaxyproject"
@@ -8,6 +6,6 @@ PROJECT_URL = "https://github.com/galaxyproject/galaxy"
 PROJECT_AUTHOR = 'Galaxy Project and Community'
 PROJECT_DESCRIPTION = 'Galaxy Datatype Framework and Datatypes'
 PROJECT_EMAIL = 'galaxy-committers@lists.galaxyproject.org'
-RAW_CONTENT_URL = "https://raw.github.com/%s/%s/master/" % (
+RAW_CONTENT_URL = "https://raw.github.com/{}/{}/master/".format(
     PROJECT_USERAME, PROJECT_NAME
 )
